library kailleraclient;

{ Important note about DLL memory management: ShareMem must be the
  first unit in your library's USES clause AND your project's (select
  Project-View Source) USES clause if your DLL exports any procedures or
  functions that pass strings as parameters or function results. This
  applies to all strings passed to and from your DLL--even those that
  are nested in records and classes. ShareMem is the interface unit to
  the BORLNDMM.DLL shared memory manager, which must be deployed along
  with your DLL. To avoid using BORLNDMM.DLL, pass string information
  using PChar or ShortString parameters. }

uses
  Themes,
  Forms,
  Windows,
  SysUtils,
  Classes,
  OpenKCore in 'source\OpenKCore.pas',
  uCore in 'source\uCore.pas',
  uKailleraClient in 'source\uKailleraClient.pas',
  ServerLists in 'source\ClientUI\ServerLists.pas',
  UIActions in 'source\ClientUI\UIActions.pas',
  uIntfCommon in 'source\ClientUI\uIntfCommon.pas',
  uSettings in 'source\ClientUI\uSettings.pas',
  uUserInterface in 'source\ClientUI\uUserInterface.pas',
  uAboutBox in 'source\ClientUI\Forms\uAboutBox.pas' {AboutBox},
  uAddrInputDialog in 'source\ClientUI\Forms\uAddrInputDialog.pas' {AddrInputDialog},
  uAddServerDialog in 'source\ClientUI\Forms\uAddServerDialog.pas' {AddServerDialog},
  uInServerDialog in 'source\ClientUI\Forms\uInServerDialog.pas' {InServerDialog},
  uOptionDialog in 'source\ClientUI\Forms\uOptionDialog.pas' {OptionDialog},
  uRoomForm in 'source\ClientUI\Forms\uRoomForm.pas' {RoomForm},
  uSelectServerDialog in 'source\ClientUI\Forms\uSelectServerDialog.pas' {SelectServerDialog},
  UWaitingWindow in 'source\ClientUI\Forms\uWaitingWindow.pas' {WaitingWindow},
  CommonUtils in '..\common\CommonUtils.pas',
  Containers in '..\common\Containers.pas',
  getvsn in '..\common\getvsn.pas',
  Instructions2 in '..\common\Instructions2.pas',
  KailleraUtils in '..\common\KailleraUtils.pas',
  Log in '..\common\Log.pas';

{$R *.res}

type
  pkailleraInfos = ^kailleraInfos;
  kailleraInfos = record
    appName: PAnsiChar;
    gameList: PAnsiChar;
    gameCallback: function(game: PAnsiChar; player: integer; numplayers: integer): integer; stdcall;
    chatReceivedCallback: procedure(nick: PAnsiChar; text: PAnsiChar); stdcall;
    clientDroppedCallback: procedure(nick: PAnsiChar; playernb: integer); stdcall;
    moreInfosCallback: procedure(gamename: PAnsiChar); stdcall;
  end;

procedure kailleraGetVersion(version: PAnsiChar); stdcall;
begin
  strcopy(version, '0.9');
  writeln('kaillera client version: ', version);
end;

procedure CoreSelectProc(ACoreIndex: integer);
begin
  case acoreindex of
    0: core := tkailleraclient.Create;
    1: core := topenkcore.Create;
  end;
	Core.setGameCreated(ui.GameCreated);
  Core.setGameClosed(ui.GameClosed);
  Core.setMyGameJoined(ui.MyGameJoined);
	Core.setMyGameLeft(ui.MyGameLeft);
	Core.setGameStarted(ui.GameStarted);
  Core.setGameStopped(ui.GameStopped);
	Core.setGameDropped(ui.GameDropped);
  Core.setChated(ui.Chated);
  Core.setGameChated(ui.GameChated);
  Core.setLoginRespond(ui.LoginRespond);
  Core.setServerMessaged(ui.ServerMessaged);
  Core.setUserJoined(ui.UserJoined);
  Core.setUserLeft(ui.UserLeft);
  Core.setUserAdd(ui.UserAdd);
  Core.setGameAdd(ui.GameAdd);
  Core.setGameStatusChanged(ui.GameStatusChanged);
  Core.setPlayerJoined(ui.PlayerJoined);
  Core.setPlayerLeft(ui.PlayerLeft);
  Core.setPlayerAdd(ui.PlayerAdd);
  Core.setNetSyncWait(ui.NetSyncWait);
  core.setKickedOut(ui.KickedOut);
end;

procedure InitializeClient; stdcall;
var
  buf: array[0..260] of char;
  path: string;
begin
{$IFDEF DEBUG}
//  reportmemoryleaksonshutdown := true;
  AllocConsole;
//  assignfile(output, ExtractFilePath(Application.ExeName)+'freekaillera.log');
//  rewrite(output);
{$ELSE}
  AssignFile(Output, 'nul');
{$ENDIF}
  trace;
  ui := TUserInterface.create;
  ui.SetCoreSelectproc(coreselectproc);
  ui.AddCoreIndex(' īϷ', 0);
  getmodulefilename(hinstance, buf, sizeof(buf));
  path := extractfilepath(buf);
  path := path+'open_kcore.dll';
  if fileexists(path) then
    ui.AddCoreIndex(' īϷ', 1);
end;

procedure Chat(text: pansichar); stdcall;
begin

  trace;
  Core.Chat(text);
end;

function SyncData(values: pansichar; size: integer): integer; stdcall;
begin

  result := Core.SyncData(values, size);

end;

procedure SetInfos(kaillerainfos: pkaillerainfos); stdcall;
var
  ac:TAppCallbacks;
begin
  trace;
  ui.SetGameList(kaillerainfos.gameList);
  ui.SetAppName(kaillerainfos.appName);
  ac.gameCallback := kaillerainfos.gameCallback;
  ac.chatReceivedCallback:= kaillerainfos.chatReceivedCallback;
  ac.clientDroppedCallback:= kaillerainfos.clientDroppedCallback;
  ac.moreInfosCallback:= kaillerainfos.moreInfosCallback;
	ui.SetAppCallbacks(ac);
end;

procedure StartAll(parent: cardinal); stdcall;
begin
  trace;
  Application.Handle := parent;
  ui.ShowMainForm;
//  try
//    repeat
//      application.ProcessMessages;
//      sleep(1);
//      core.ProcessInstruction;
//    until (not ui.MainFormShowing);
//  finally
//    try
//      ui.freeforms;
//    except
//      on E: Exception do writeln(e.Message);
//    end;
//  end;
//  if parent <> 0 then
//    while not setforegroundwindow(parent) do;
end;

procedure TerminateGame; stdcall;
begin
  trace;
  Core.TerminateGame;
end;

procedure FinalizeClient; stdcall;
begin
{$IFDEF DEBUG}
//  closefile(output);
{$ENDIF}
  core := nil;
  ui.free;
{$IFDEF VER200}
  themeservices.free;
{$ENDIF}
  trace;
end;

exports
  kailleraGetVersion name '_kailleraGetVersion@4',
  InitializeClient name '_kailleraInit@0',
  FinalizeClient name '_kailleraShutdown@0',
  SetInfos name '_kailleraSetInfos@4',
  StartAll name '_kailleraSelectServerDialog@4',
  SyncData name '_kailleraModifyPlayValues@8',
  Chat name '_kailleraChatSend@4',
  TerminateGame name '_kailleraEndGame@0';

begin
end.
