unit uInServerDialog;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, Buttons, StdCtrls, ComCtrls, JvExStdCtrls, JvRichEdit,
  ExtCtrls;

type
  TInServerDialog = class(TForm)
    IdleRoom: TGroupBox;
    UserListView: TListView;
    ChatingBar: TEdit;
    ChatingButton: TButton;
    GameListGroup: TGroupBox;
    GameListView: TListView;
    MakeGameButton: TButton;
    JoinGameButton: TButton;
    BitBtn1: TBitBtn;
    GameNamePopup: TPopupMenu;
    ChatingScreen: TJvRichEdit;
    SpeedButton1: TSpeedButton;
    Splitter1: TSplitter;
    procedure MakeGameButtonClick(Sender: TObject);
    procedure JoinGameButtonClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ChatingButtonClick(Sender: TObject);
    procedure ChatingBarKeyPress(Sender: TObject; var Key: Char);
    procedure GameListViewDblClick(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure ChatingScreenURLClick(Sender: TObject; const URLText: string;
      Button: TMouseButton);
    procedure GameListViewCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
    procedure GameListViewColumnClick(Sender: TObject; Column: TListColumn);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure UserListViewColumnClick(Sender: TObject; Column: TListColumn);
    procedure UserListViewCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
    procedure SpeedButton1Click(Sender: TObject);
    procedure GameListViewKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure BitBtn1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    FColumnToSort: integer;
    FAssending: boolean;
    FUserListSortAssending: boolean;
    procedure PopupClick(Sender: TObject);
    procedure WMACTIVATE(var message: tmessage); message WM_ACTIVATE;
    procedure SetGameList;
  public
    { Public declarations }
    property columntosort: integer  read fcolumntosort write fcolumntosort;
  end;

var
  InServerDialog: TInServerDialog;

implementation

uses uRoomForm, ShellApi, clipbrd,
  uAboutBox, uCore, uselectserverdialog, uintfcommon;

{$R *.dfm}

procedure TInServerDialog.BitBtn1Click(Sender: TObject);
begin
  Close;
end;

procedure TInServerDialog.BitBtn1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (button = mbmiddle) and ([ssalt, ssctrl] <= shift) then
    core.BreakServer;
end;

procedure TInServerDialog.ChatingBarKeyPress(Sender: TObject; var Key: Char);
begin
	if Key = #13 then
	begin
		ChatingButtonClick(Sender);
		Key := #0;
	end
end;

procedure TInServerDialog.ChatingButtonClick(Sender: TObject);
begin
  if ChatingBar.Text = '' then exit;
  Core.SendChat(ChatingBar.Text);
  ChatingBar.Text :='';
  ChatingBar.SetFocus();
end;

procedure TInServerDialog.ChatingScreenURLClick(Sender: TObject;
  const URLText: string; Button: TMouseButton);
begin
	ShellExecute(Handle, 'open', pchar(URLText), nil, nil, SW_SHOWNORMAL);
end;

procedure TInServerDialog.WMACTIVATE(var message: tmessage);
begin
  if (message.WParam = WA_ACTIVE) or (message.WParam = WA_CLICKACTIVE) then
    if RoomForm.Visible then RoomForm.SetFocus;
end;

procedure TInServerDialog.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if RoomForm.Showing then
    RoomForm.Close;
  selectserverdialog.ConnStatEdit.Text := '';
  core.Logout(intfcommon.LeaveMessage);
end;

procedure TInServerDialog.FormShow(Sender: TObject);
begin
	ChatingScreen.Clear();
	UserListView.Clear();
	GameListView.Clear();
	RoomForm.Top := self.Top + 500;
	RoomForm.Left := self.Left+200;
	ChatingBar.SetFocus();
  setgamelist;
end;

procedure TInServerDialog.GameListViewColumnClick(Sender: TObject;
  Column: TListColumn);
begin
  FAssending := not FAssending;
  FColumnToSort := Column.Index;
  GameListView.AlphaSort;
end;

procedure TInServerDialog.GameListViewCompare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer);
var s1, s2: string;
begin
  if FColumnToSort = 0 then
  begin
    s1 := item1.Caption;
    s2 := item2.Caption;
  end
  else
  begin
    s1 := item1.subitems[FColumnToSort-1];
    s2 := item2.SubItems[FColumnToSort-1];
  end;
  compare := comparetext(s1, s2);
  if not FAssending then compare := compare * -1;
end;

procedure TInServerDialog.GameListViewDblClick(Sender: TObject);
begin
  JoinGameButtonClick(Sender);
end;

procedure TInServerDialog.GameListViewKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if key = vk_return then joingamebuttonclick(sender);
end;

procedure TInServerDialog.JoinGameButtonClick(Sender: TObject);
  function HasTitle(AGameTitle: string): boolean;
  var I, j: integer;
  begin
    result := false;
    for I := 0 to GameNamePopup.Items.Count - 1 do
    begin
      if gamenamepopup.Items.Items[i].Caption = agametitle then
      begin
        result := true;
        exit;
      end;
      for j:= 0 to GameNamePopup.Items.Items[i].Count-1 do
        if GameNamePopup.Items.Items[i].Items[j].Caption = AGameTitle then
        begin
          result := true;
          exit;
        end;
    end;
  end;
begin
	if not RoomForm.Showing then // InGame
		if GameListView.ItemIndex <> -1 then
		begin
      if not HasTitle(GameListView.Selected.Caption) then
        ShowMessage(' ϴ: '+''''+GameListView.Selected.Caption+'''')
      else
        if GameListView.Selected.SubItems.Strings[2] = '' then
          ShowMessage(' 濡   ϴ.')
        else
        begin
          RoomForm.InGameTitleEdit.Text := GameListView.Selected.Caption;
          RoomForm.GameStartButton.Enabled := false;
          RoomForm.KickButton.Enabled := false;
          Core.JoinGame(integer(GameListView.Selected.Data));
          writeln('濡 , ID: ', integer(GameListView.Selected.Data));
        end;
		end
end;

procedure TInServerDialog.MakeGameButtonClick(Sender: TObject);
var
  x, y: integer;
begin
	x := self.Left + GameListGroup.Left + MakeGameButton.Left;
	y := self.Top + GameListGroup.Top + MakeGameButton.Top;
	GameNamePopup.Popup(x, y);
end;

procedure TInServerDialog.PopupClick(Sender: TObject);
var mi: TMenuItem;
begin
  mi := TMenuItem(Sender);
  RoomForm.InGameTitleEdit.Text := mi.Caption;
  RoomForm.GameStartButton.Enabled := true;
  RoomForm.KickButton.Enabled := true;
  Core.creategame(mi.Caption);
end;

procedure TInServerDialog.SetGameList;
var
  i:integer;
  mi, parent: TMenuItem;
  sl: tstringlist;
  first_char: char;
  idx: integer;
begin
  gamenamepopup.Items.Clear;
  sl := tstringlist.Create;
  for I := 0 to intfcommon.GameList.Count - 1 do
  begin
    mi := TMenuItem.Create(self);
    mi.Caption := intfcommon.gamelist.Strings[i];
    mi.OnClick := PopupClick;
    first_char := mi.Caption[1]; // ù ڷ  .
    if sl.IndexOf(first_char) = -1 then
    begin
      idx := sl.Addobject(first_char, tmenuitem.Create(self));
      parent := tmenuitem(sl.Objects[idx]);
      parent.Caption := first_char;
    end;
    idx := sl.IndexOf(first_char);
    parent := tmenuitem(sl.Objects[idx]);
    parent.Add(mi);
  end;
  { äù ϳ  տ  ߰ }
  mi := tmenuitem.Create(self);
  mi.Caption := '*Chat (not game)';
  mi.OnClick := PopupClick;
  gamenamepopup.Items.Add(mi);
  {  ߰ }
  for i:= 0 to sl.Count-1 do
    gamenamepopup.Items.Add(tmenuitem(sl.Objects[i]));
  sl.Free;
end;

procedure TInServerDialog.SpeedButton1Click(Sender: TObject);
begin
  if roomform.Showing then
    roomform.SetFocus;
end;

procedure TInServerDialog.UserListViewColumnClick(Sender: TObject;
  Column: TListColumn);
begin
  fuserlistsortassending := not fuserlistsortassending;
  fcolumntosort := column.Index;
  userlistview.AlphaSort;
end;

procedure TInServerDialog.UserListViewCompare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer);
var
  s1, s2: string;
  i1, i2: integer;
begin
  if FColumnToSort = 0 then
  begin
    s1 := item1.Caption;
    s2 := item2.Caption;
  end
  else
  begin
    s1 := item1.subitems[FColumnToSort-1];
    s2 := item2.SubItems[FColumnToSort-1];
  end;
  if fcolumntosort <> 1 then
    compare := comparetext(s1, s2)
  else begin
    s1 := stringreplace(s1, ' ms', '', []);
    s2 := stringreplace(s2, ' ms', '', []);
    i1 := strtoint(s1);
    i2 := strtoint(s2);
    if i1 > i2 then compare := -1
    else if i1 = i2 then compare := 0
    else compare := 1;
  end;
  if not Fuserlistsortassending then compare := compare * -1;
end;

end.
