unit uSelectServerDialog;

interface

uses
   Forms, StdCtrls, ExtCtrls, Controls, Buttons, ComCtrls, Classes,
   Messages, ServerLists;

type
  TSelectServerDialog = class(TForm)
    pcServerLists: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    OfficialServerListView: TListView;
    FavServerListView: TListView;
    VisitServerListView: TListView;
    Button1: TButton;
    Button3: TButton;
    BitBtn3: TBitBtn;
    Panel1: TPanel;
    Label1: TLabel;
    Button2: TButton;
    UserNameEdit: TLabeledEdit;
    ConnTypeComboBox: TComboBox;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    ConnectButton: TBitBtn;
    OfficialListStatusLabel: TLabel;
    ConnStatEdit: TEdit;
    TabSheet4: TTabSheet;
    UnofficialServerListView: TListView;
    UnofficialListStatusLabel: TLabel;
    BitBtn4: TBitBtn;
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    Button9: TButton;
    Button10: TButton;
    BitBtn5: TBitBtn;
    ProgramNameLabel: TLabel;
    Timer1: TTimer;
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button3Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure OfficialServerListViewKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure Button1Click(Sender: TObject);
    procedure BitBtn3Click(Sender: TObject);
    procedure BitBtn4Click(Sender: TObject);
    procedure OfficialServerListViewDblClick(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure Button9Click(Sender: TObject);
    procedure pcServerListsChange(Sender: TObject);
    procedure Button10Click(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure ConnectButtonClick(Sender: TObject);
    procedure OfficialServerListViewSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure OfficialServerListViewCustomDrawItem(Sender: TCustomListView;
      Item: TListItem; State: TCustomDrawState; var DefaultDraw: Boolean);
    procedure BitBtn5Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    FFavoriteServerList: TFavoriteServerList;
    FVisitServerList: TVisitServerList;
    FOfficialServerList: TOfficialServerList;
    FUnofficialServerList: TUnofficialServerList;
    FServerName, FAddress: string;
    FCurrentServerList: TServerList;
    procedure ConnectProcess(ServerName, Address: string);
    procedure ClearAllPinging;
    procedure WMACTIVATE(var message: tmessage); message WM_ACTIVATE;
    procedure OfficialListStatus(const Status: string);
    procedure UnOfficialListStatus(const Status: string);
    procedure HandleInstruction(var message: tmessage); message wm_user;
  end;

var
  SelectServerDialog: TSelectServerDialog;

implementation

uses
  SysUtils, Graphics, windows, strutils,
  uInServerDialog, uAboutBox, uOptionDialog, uAddrInputDialog,
  uAddServerDialog, usettings, uintfcommon, ucore, log, uiactions;

{$R *.dfm}

type
  TConnectThread = class(TThread)
  private
    FAddress: string;
  protected
    procedure Execute; override;
  public
    constructor Create(Address: string);
  end;

procedure AddressSeperator(const Address: string; var Host: string; var Port: integer);
var
  colon: integer;
begin
	colon := Pos(':', Address);
	if colon <> 0 then
	begin
		host := midstr(Address, 1, colon-1);
		port := StrToIntDef(MidStr(Address, colon+1, Length(Address)), 27888);
	end
	else
	begin
		host := Address;
		port := 27888;
	end;
end;

procedure TSelectServerDialog.FormShow(Sender: TObject);
begin
  programnamelabel.Caption := intfcommon.programtitle+' '+intfcommon.getrversion;
	UserNameEdit.Text := Settings.UserName;
	ConnTypeComboBox.ItemIndex := Settings.ConnType-1;
  FFavoriteServerList := TFavoriteServerList.Create(FavServerListView);
  FVisitServerList := TVisitServerList.Create(VisitServerListView);
  FOfficialServerList := TOfficialServerList.Create(OfficialServerListView);
  fofficialserverlist.SetStatusProc(officialliststatus);
  FUnofficialServerList := TUnofficialServerList.Create(UnofficialServerListView);
  funofficialserverlist.SetStatusProc(unofficialliststatus);
  FFavoriteServerList.Load;
  FVisitServerList.Load;
  pcServerLists.ActivePageIndex := Settings.ActivePage;
  pcServerListsChange(Sender);
  actionmanager := tactionmanager.Create;
end;

procedure TSelectServerDialog.HandleInstruction(var message: tmessage);
begin
  trace;
end;

procedure TSelectServerDialog.OfficialListStatus(const Status: string);
begin
  officialliststatuslabel.Caption := status;
  application.ProcessMessages;
end;

procedure TSelectServerDialog.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  actionmanager.Free;
  FVisitServerList.Save;
  FFavoriteServerList.Save;
  FVisitServerList.Free;
  FFavoriteServerList.Free;
  FOfficialServerList.Free;
  FUnofficialServerList.Free;
  if inserverdialog.Showing then inserverdialog.Close;
end;

procedure TSelectServerDialog.BitBtn1Click(Sender: TObject);
begin
  AboutBox.ShowModal;
end;

procedure TSelectServerDialog.BitBtn2Click(Sender: TObject);
begin
  close;
end;

procedure TSelectServerDialog.BitBtn3Click(Sender: TObject);
begin
  FOfficialServerList.TerminatePinging;
  FOfficialServerList.Load;
end;

procedure TSelectServerDialog.BitBtn4Click(Sender: TObject);
begin
  FUnofficialServerList.TerminatePinging;
  FUnofficialServerList.Load;
end;

procedure TSelectServerDialog.BitBtn5Click(Sender: TObject);
begin
  if optiondialog.ShowModal = mrok then
  begin
    intfcommon.CoreSelector.SelectCore(
      integer(optiondialog.CoreSelectComboBox.Items.Objects[optiondialog.CoreSelectComboBox.ItemIndex])
    );
    intfcommon.LeaveMessage := optiondialog.LeaveMessageEdit.Text;
    intfcommon.LocationTexts := optiondialog.LocationTextsEdit.Text;
    intfcommon.ShowingUserInOut := optiondialog.showinguserinoutcheckbox.Checked;
  end;
end;

procedure TSelectServerDialog.Button10Click(Sender: TObject);
begin
  FCurrentServerList.TerminatePinging;
  FCurrentServerList.StartPinging;
end;

procedure TSelectServerDialog.Button1Click(Sender: TObject);
begin
  if FavServerListView.ItemIndex <> -1 then
  begin
    TSavedServerList(FFavoriteServerList).DeleteServer(FavServerListView.Selected.Index);
  end;
end;

procedure TSelectServerDialog.Button2Click(Sender: TObject);
begin
  with TControl(Sender) do
  begin
    AddrInputDialog.Left := self.Left + parent.Left + left;
    AddrInputDialog.Top := self.Top + parent.Top + top;
  end;
	if AddrInputDialog.ShowModal() = mrOk then
  begin
    ConnectProcess('()', AddrInputDialog.AddrInput.Text);
  end;
end;

procedure TSelectServerDialog.Button3Click(Sender: TObject);
begin
  if AddServerDialog.ShowModal = mrOk then
  begin
    TSavedServerList(FFavoriteServerList).AddServer(AddServerDialog.IpAddr.Text, AddServerDialog.ServerName.Text);
  end;
end;

procedure TSelectServerDialog.Button4Click(Sender: TObject);
begin
  if OfficialServerListView.ItemIndex <> -1 then
    with OfficialServerListView.Selected do
    begin
      tsavedserverlist(ffavoriteserverlist).AddServer(subitems[0], caption);
    end;
end;

procedure TSelectServerDialog.Button5Click(Sender: TObject);
begin
  tsavedserverlist(ffavoriteserverlist).DeleteAll;
end;

procedure TSelectServerDialog.Button6Click(Sender: TObject);
begin
  if unofficialserverlistview.ItemIndex <> -1 then
    with unofficialserverlistview.Selected do
    begin
      tsavedserverlist(ffavoriteserverlist).AddServer(subitems[0], caption);
    end;
end;

procedure TSelectServerDialog.Button9Click(Sender: TObject);
begin
  TSavedServerList(FVisitServerList).DeleteAll;
end;

procedure TSelectServerDialog.ClearAllPinging;
begin
  FFavoriteServerList.TerminatePinging;
  FVisitServerList.TerminatePinging;
  FOfficialServerList.TerminatePinging;
  FUnofficialServerList.TerminatePinging;
end;

procedure TSelectServerDialog.ConnectButtonClick(Sender: TObject);
begin
  connectprocess(fservername, faddress);
end;

procedure TSelectServerDialog.ConnectProcess(ServerName, Address: string);
begin
  clearallpinging;
  if servername = '' then exit;
  connstatedit.Text := '';
  sleep(100);
  ConnStatEdit.Font.Color := clBlack;
  ConnStatEdit.Text := ' õ... ';
  application.ProcessMessages;
  sleep(100); //   ̱ 
  inserverdialog.Caption := servername;
  tsavedserverlist(fvisitserverlist).AddServer(address, servername);
  Core.Login(intfcommon.AppName, UsernameEdit.Text, Address, ConnTypeComboBox.ItemIndex+1);
end;

procedure TSelectServerDialog.WMACTIVATE(var message: tmessage);
begin
  if (message.WParam = WA_ACTIVE) or (message.WParam = WA_CLICKACTIVE) then
  begin
    if addrinputdialog.Visible then addrinputdialog.SetFocus;
    if optiondialog.Visible then optiondialog.SetFocus;
  end;
end;

procedure TSelectServerDialog.OfficialServerListViewCustomDrawItem(
  Sender: TCustomListView; Item: TListItem; State: TCustomDrawState;
  var DefaultDraw: Boolean);
begin
  if integer(item.Data) = 17 then
    sender.Canvas.Brush.Color := $ffc8c8
  else
    sender.Canvas.Brush.Color := clwindow;
end;

procedure TSelectServerDialog.OfficialServerListViewDblClick(Sender: TObject);
var item: tlistitem;
begin
  item := tlistview(sender).Selected;
  if item = nil then exit;
  FServerName := Item.Caption;
  FAddress := Item.SubItems.Strings[0];
  ConnectProcess(FServerName, FAddress);
end;

procedure TSelectServerDialog.OfficialServerListViewKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_RETURN: ConnectButtonClick(Sender);
  end;
end;

procedure TSelectServerDialog.OfficialServerListViewSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  FServerName := Item.Caption;
  FAddress := Item.SubItems.Strings[0];
end;

procedure TSelectServerDialog.pcServerListsChange(Sender: TObject);
begin
  clearallpinging;
  case pcServerLists.ActivePageIndex of
  0: FCurrentServerList := FOfficialServerList;
  1: FCurrentServerList := FUnofficialServerList;
  2: FCurrentServerList := FFavoriteServerList;
  3: FCurrentServerList := FVisitServerList;
  end;
  Settings.ActivePage := pcServerLists.ActivePageIndex;
end;

procedure TSelectServerDialog.Timer1Timer(Sender: TObject);
begin
  actionmanager.ProcessActions;
end;

procedure TSelectServerDialog.UnOfficialListStatus(const Status: string);
begin
  unofficialliststatuslabel.Caption := status;
  unofficialliststatuslabel.Update;
end;

{ TConnectThread }

constructor TConnectThread.Create(Address: string);
begin
  inherited Create(false);
  FAddress := Address;
end;

procedure TConnectThread.Execute;
//var host: string; port: integer;
begin
  inherited;
  returnvalue := 0;
//  if not Core.core_initialize(0, pansichar(ansistring(SelectServerDialog.FAppName)), pansichar(ansistring(SelectServerDialog.UserNameEdit.Text)), SelectServerDialog.conntypecombobox.ItemIndex+1) then exit;
//  AddressSeperator(FAddress, host, port);
//  if Core.core_connect(pansichar(ansistring(host)), port) then
//  begin
//    returnvalue := 1;
//  end;
end;

end.
