unit UIActions;

interface

uses Graphics, SyncObjs, Contnrs;

type

  TAction = class
    procedure DoAction; virtual; abstract;
  end;

  TEnterGame = class(TAction)
    procedure DoAction; override;
  end;

  TExitGame = class(TAction)
    procedure DoAction; override;
  end;

  TChatUp = class(TAction)
    Message: string;
    Color: TColor;
    procedure DoAction; override;
  end;

  TGameChatUp = class(TAction)
    Message: string;
    Color: TColor;
    procedure DoAction; override;
  end;

  TSetUser = class(TAction)
    Name: string;
    Ping, Status, ID, Conn: integer;
    procedure DoAction; override;
  end;

  TRemoveUser = class(TAction)
    UserID: integer;
    procedure DoAction; override;
  end;

  TSetGame = class(TAction)
    Title: string;
    ID: integer;
    App: string;
    Owner: string;
    Users: string;
    Status: integer;
    procedure DoAction; override;
  end;

  TRemoveGame = class(TAction)
    GameID: integer;
    procedure DoAction; override;
  end;

  TSetPlayer = class(TAction)
    UserID: integer;
    procedure DoAction; override;
  end;

  TRemovePlayer = class(TAction)
    UserID: integer;
    procedure DoAction; override;
  end;

  TStartGame = class(TAction)
    procedure DoAction; override;
  end;

  TStopGame = class(TAction)
    procedure DoAction; override;
  end;

  TEnterServer = class(TAction)
    procedure DoAction; override;
  end;

  TConnStat = class(TAction)
    Msg: string; Color: TColor;
    procedure DoAction; override;
  end;

  TActionManager = class
  private
    CS: TCriticalSection;
    ActionQueue: TObjectQueue;
  public
    constructor Create;
    destructor Destroy; override;
    procedure PerformAction(AAction: TAction);
    procedure ProcessActions;
  end;

var
  ActionManager: TActionManager;

implementation

uses
  Windows, Messages, comctrls, sysutils, forms, log,
  uSelectServerDialog, uInServerDialog, uAboutBox, uAddrInputDialog,
  uAddServerDialog, uRoomForm, uSettings, uoptiondialog;

const
  GAME_STATUS: array[0..2] of string =
  (
    'ٸ',
    '', // ݽũ
    ''
  );

  CONNECTION_TYPES: array[0..6] of string =
  (
    '̻',
    '',
    '',
    '',
    '',
    '',
    ''
  );

  USER_STATUS: array[0..2] of string =
  // ĸĿ 0~1, īϷ 1~2
  (
    '',
    '',
    ''
  );

function GetItemByID(AListItems: TListItems; ID: integer): TListItem;
var i:integer;
begin
  for i:= 0 to alistitems.count-1 do
  begin
    if integer(alistitems.item[i].data) = id then
    begin
      result := alistitems.item[i];
      exit;
    end;
  end;
  result := nil;
end;

var
  GGameStartButtonStatus: boolean;
  GStayTopStatus: TFormStyle;
  GServerName: string;

{ TActionManager }

constructor TActionManager.Create;
begin
  cs := tcriticalsection.Create;
  ActionQueue := tobjectqueue.Create;
end;

destructor TActionManager.Destroy;
begin
  ActionQueue.Free;
  cs.Free;
  inherited;
end;

procedure TActionManager.PerformAction(AAction: TAction);
begin
  cs.Acquire;
  try
    actionqueue.Push(aaction);
  finally
    cs.Release;
  end;
end;

procedure TActionManager.ProcessActions;
var i: integer;
begin
  cs.Acquire;
  try
    i:= 0;
    while i < actionqueue.Count do
    begin
      with taction(actionqueue.Pop) do
      begin
        DoAction;
        free;
      end;
      inc(i);
    end;
  finally
    cs.Release;
  end;
end;

{ TEnterGame }

procedure TEnterGame.DoAction;
begin
  inserverdialog.makegamebutton.enabled := false;
  inserverdialog.joingamebutton.enabled := false;
  roomform.show;
end;

{ TExitGame }

procedure TExitGame.DoAction;
begin
  inserverdialog.makegamebutton.enabled := true;
  inserverdialog.joingamebutton.enabled := true;
  if roomform.showing then roomform.Close();
end;

{ TChatUp }

procedure TChatUp.DoAction;
begin
	InServerDialog.ChatingScreen.SelAttributes.Color := color;
	InServerDialog.ChatingScreen.Lines.Add(Message);
//	SendMessage(InServerDialog.ChatingScreen.Handle, WM_VSCROLL, SB_BOTTOM, 0);
end;

{ TGameChatUp }

procedure TGameChatUp.DoAction;
begin
	roomform.gamechatingscreen.SelAttributes.Color := Color;
  roomform.gamechatingscreen.lines.append(Message);
	SendMessage(RoomForm.GameChatingScreen.Handle, WM_VSCROLL, SB_BOTTOM, 0);
end;

{ TSetUser }

procedure TSetUser.DoAction;
var
  li: tlistitem;
begin
  li :=	GetItemById(InServerDialog.UserListView.Items, id);
  if li = nil then
  begin
    li := InServerDialog.UserListView.Items.Add();
    li.Data := pointer(id);
    li.Caption := name;
    li.SubItems.Add('');
    li.SubItems.Add('');
    li.SubItems.Add('');
  end;
  li.SubItems.Strings[0] := inttostr(ping)+' ms';
  li.SubItems.Strings[1] := CONNECTION_TYPES[conn];
  li.SubItems.Strings[2] := USER_STATUS[status];
  inserverdialog.Caption := format(gservername+' (%d)',[inserverdialog.UserListView.Items.Count]);
end;

{ TRemoveUser }

procedure TRemoveUser.DoAction;
var li: tlistitem;
begin
	li := GetItemById(InServerDialog.UserListView.Items, userid);
	if li <> nil then li.Delete;
  inserverdialog.Caption := format(gservername+' (%d)',[inserverdialog.UserListView.Items.Count]);
end;

{ TSetGame }

procedure TSetGame.DoAction;
var li:tlistitem;
begin
  li := getitembyid(inserverdialog.GameListView.Items, ID);
  if not assigned(li) then
  begin
    li := inserverdialog.GameListView.Items.Add;
    with li do
    begin
      Data:=pointer(ID);
      Caption:=Title;
      subitems.Add('');
      subitems.Add('');
      subitems.Add('');
      subitems.Add('');
    end;
  end;
  if App <> '' then // for gamestatus
  begin
    li.SubItems.Strings[0] := App;
    li.SubItems.Strings[1] := Owner;
  end;
  li.SubItems.Strings[2] := GAME_STATUS[Status];
  li.SubItems.Strings[3] := Users;
end;

{ TRemoveGame }

procedure TRemoveGame.DoAction;
var
  li: tlistitem;
begin
	li := GetItemById(InServerDialog.GameListView.Items, gameid);
	if li <> nil then li.Delete;
end;

{ TSetPlayer }

procedure TSetPlayer.DoAction;
var li, added:tlistitem;
begin
	li := GetItemById(InServerDialog.UserListView.Items, userid);
	if li <> nil then
  begin
		added := RoomForm.PlayerListView.Items.Add();
		added.Data := pointer(userid);
		added.Caption := li.Caption;
		if (added.Caption <> SelectServerDialog.UserNameEdit.Text) and RoomForm.chkJoinRing.Checked then Beep;
		added.SubItems.Add(li.SubItems.Strings[0]);
		added.SubItems.Add(li.SubItems.Strings[1]);
  end;
end;

{ TRemovePlayer }

procedure TRemovePlayer.DoAction;
var
  li:tlistitem;
begin
	li := GetItemById(RoomForm.PlayerListView.Items, userid);
	if li <> nil then li.Delete;
end;

{ TStartGame }

procedure TStartGame.DoAction;
begin
  Ggamestartbuttonstatus := roomform.GameStartButton.Enabled;
	RoomForm.GameStartButton.Enabled := false;
	RoomForm.ExitButton.Enabled := false;
  gstaytopstatus := roomform.FormStyle;
  roomform.FormStyle := fsnormal;
end;

{ TStopGame }

procedure TStopGame.DoAction;
begin
	RoomForm.GameStartButton.Enabled := ggamestartbuttonstatus;
	RoomForm.ExitButton.Enabled := true;
  roomform.FormStyle := gstaytopstatus;
end;

{ TEnterServer }

procedure TEnterServer.DoAction;
begin
  GServername := inserverdialog.Caption;
  // same as 'exit game'
  inserverdialog.makegamebutton.enabled := true;
  inserverdialog.joingamebutton.enabled := true;
  if roomform.showing then roomform.Close();
  //
  Settings.UserName := selectserverdialog.UserNameEdit.Text;
  Settings.ConnType := selectserverdialog.ConnTypeComboBox.ItemIndex+1;
  InServerDialog.Show();
end;

{ TConnStat }

procedure TConnStat.DoAction;
begin
  selectserverdialog.ConnStatEdit.Font.Color := color;
  selectserverdialog.ConnStatEdit.Text := msg;
end;

end.
