unit uIntfCommon;

interface

uses Classes;

type
  TCoreSelector = class;

  TIntfCommon = class
  private
    FGameList: TStringList;
    FCoreSelector: TCoreSelector;
    function GetLeaveMessage: string;
    procedure SetLeaveMessage(LeaveMessage: string);
    function GetLocaionTexts: string;
    procedure SetLocationTexts(const Value: string);
    procedure SetShowingUserInOut(const Value: boolean);
    function GetShowingUserInOut: boolean;
  public
    AppName: string;
    constructor Create;
    destructor Destroy; override;
    function GetAllVersion: string;
    function GetRVersion: string;
    property LeaveMessage: string read getleavemessage write setleavemessage;
    property CoreSelector: TCoreSelector read FCoreSelector;
    property GameList: TStringList read FGameList;
    property LocationTexts: string read GetLocaionTexts write SetLocationTexts;
    property ShowingUserInOut: boolean read GetShowingUserInOut write SetShowingUserInOut;
    function ProgramTitle: string;
    function AuthorUrl: string;
  end;

  TCoreSelectProc = procedure(AItemIndex: integer);
  TCoreSelector = class
  private
    FSelectedCoreIndex: integer;
    FCoreIndexList: TStringList;
    FCoreSelectProc: TCoreSelectProc;
    FCoreInitialized: boolean;
  public
    constructor Create;
    destructor Destroy; override;
    procedure SetCoreSelectProc(ACoreSelectProc: TCoreSelectProc);
    procedure AddCoreIndex(ACoreName: string; ACoreIndex: integer);
    procedure SelectCore(CoreIndex: integer);
    function CoreCount: integer;
    function GetCoreInfo(Index: integer; out CoreIndex: integer): string;
    property SelectedCoreIndex: integer read FSelectedCoreIndex;
    procedure InitializeCore;
  end;

var
  IntfCommon: TIntfCommon;

implementation

uses windows, sysutils, forms, getvsn, usettings, log;

const
  CProgramTitle = 'Free Kaillera Client';
  CAuthorUrl = 'http://lnl.pe.kr';

constructor TIntfCommon.Create;
begin
  FGameList := TStringList.Create;
  fcoreselector := tcoreselector.Create;
end;

destructor TIntfCommon.Destroy;
begin
  fcoreselector.free;
  FGameList.Free;
  inherited;
end;

function TIntfCommon.GetAllVersion: string;
var
  major, minor, release, build: word;
  buf: array[0..260] of char;
begin
  getmodulefilename(hinstance, buf, sizeof(buf));
	getfileversion(buf, major, minor, release, build);
  if release = 0 then begin
    if build = 0 then
    	result := format('v%d alpha', [major])
    else
      result := format('v%d beta build %d', [major, build]);
  end else
    if major <> 0 then
    	result := format('R%d build %d', [release, build])
    else
      result := format('pre-R%d build %d', [release, build]);
end;

function TIntfCommon.GetRVersion: string;
var
  major, minor, release, build: word;
  buf: array[0..260] of char;
begin
  getmodulefilename(hinstance, buf, sizeof(buf));
	getfileversion(buf, major, minor, release, build);
  if major <> 0 then
    result := format('R%d',[release])
  else
    result := format('pre-R%d',[release]);
end;

function TIntfCommon.GetShowingUserInOut: boolean;
begin
  result := settings.ShowingUserInOut;
end;

{ TCoreSelector }

procedure TCoreSelector.AddCoreIndex(ACoreName: string; ACoreIndex: integer);
begin
  fcoreindexlist.AddObject(acorename, tobject(acoreindex));
end;

function TCoreSelector.CoreCount: integer;
begin
  result := fcoreindexlist.Count;
end;

function TCoreSelector.GetCoreInfo(Index: integer; out CoreIndex: integer): string;
begin
  result := fcoreindexlist[index];
  coreindex := integer(fcoreindexlist.Objects[index]);
end;

procedure TCoreSelector.InitializeCore;
begin
  if fcoreindexlist.IndexOfObject(tobject(fselectedcoreindex)) <> -1 then
    selectcore(fselectedcoreindex)
  else
    selectcore(0);
end;

constructor TCoreSelector.Create;
begin
  fcoreindexlist := tstringlist.Create;
  fcoreinitialized := false;
  fselectedcoreindex := Settings.CoreIndex;
end;

destructor TCoreSelector.Destroy;
begin
  Settings.CoreIndex := fselectedcoreindex;
  fcoreindexlist.Free;
  inherited;
end;

procedure TCoreSelector.SelectCore(CoreIndex: integer);
begin
  if (fcoreinitialized) and (coreindex = fselectedcoreindex) then exit;
  fcoreselectproc(coreindex);
  fselectedcoreindex := coreindex;
  fcoreinitialized := true;
end;

procedure TCoreSelector.SetCoreSelectProc(ACoreSelectProc: TCoreSelectProc);
begin
  fcoreselectproc := acoreselectproc;
end;

function TIntfCommon.GetLeaveMessage: string;
begin
  result := settings.LeaveMessage;
end;

function TIntfCommon.GetLocaionTexts: string;
begin
  result := settings.LocationTexts;
end;

procedure TIntfCommon.SetLeaveMessage(LeaveMessage: string);
begin
  settings.LeaveMessage := LeaveMessage;
end;

procedure TIntfCommon.SetLocationTexts(const Value: string);
begin
  settings.LocationTexts := value;
end;

procedure TIntfCommon.SetShowingUserInOut(const Value: boolean);
begin
  settings.ShowingUserInOut := Value;
end;

function TIntfCommon.AuthorUrl: string;
begin
  result := Cauthorurl;
end;

function TIntfCommon.ProgramTitle: string;
begin
  result := Cprogramtitle;
end;

initialization
  intfcommon := tintfcommon.Create;

finalization
  intfcommon.Free;

end.
