unit uSettings;

interface

uses IniFiles, Classes;

type
  TSettings = class
  private
    FIniFile: TIniFile;
    FTempStream: TStringStream;
    function GetConnType: integer;
    function GetJoinRing: boolean;
    function GetStayTop: boolean;
    function GetSVIndex: integer;
    function GetUserName1: string;
    procedure SetConnType(const Value: integer);
    procedure SetJoinRing(const Value: boolean);
    procedure SetStayTop(const Value: boolean);
    procedure SetSVIndex(const Value: integer);
    procedure SetUserName(const Value: string);
    function GetActivePage: integer;
    procedure SetActivePage(const Value: integer);
    function GetCoreIndex: integer;
    procedure SetCoreIndex(const Value: integer);
    function GetLeaveMessage: string;
    procedure SetLeaveMessage(const Value: string);
    function GetLocationTexts: string;
    procedure SetLocationTexts(const Value: string);
    function GetShowingUserInOut: boolean;
    procedure SetShowingUserInOut(const Value: boolean);
  public
    constructor Create;
    destructor Destroy; override;
    property UserName: string read GetUserName1 write SetUserName;
    property ConnType: integer read GetConnType write SetConnType;
    property StayTop: boolean read GetStayTop write SetStayTop;
    property JoinRing: boolean read GetJoinRing write SetJoinRing;
    property SVIndex: integer read GetSVIndex write SetSVIndex;
    property ActivePage: integer read GetActivePage write SetActivePage;
    property CoreIndex: integer read GetCoreIndex write SetCoreIndex;
    property LeaveMessage: string read GetLeaveMessage write SetLeaveMessage;
    property LocationTexts: string read GetLocationTexts write SetLocationTexts;
    property ShowingUserInOut: boolean read GetShowingUserInOut write SetShowingUserInOut;
    procedure GetFavServerList(ListBuf: TStrings);
    procedure SetFavServerList(ServerList: TStrings);
    procedure GetVisitServerList(ListBuf: TStrings);
    procedure SetVisitServerList(ServerList: TStrings);
    procedure GetFavoriteServerList(AStreamBuf: TStream);
    procedure GetRoomFormPos(var X, Y: integer);
    procedure SetRoomFormPos(const X, Y: integer);
  end;

var
  Settings: TSettings;

implementation

uses SysUtils, Forms, uintfcommon;

{ TSetting }

constructor TSettings.Create;
begin
  FIniFile := TIniFile.Create(ExtractFilePath(Application.ExeName)+'kailleraclient.ini');
  FTempStream := TStringStream.Create('');
end;

destructor TSettings.Destroy;
begin
  try
    FTempStream.Free;
    FIniFile.Free;
  finally
    // pass
  end;
  inherited;
end;

function TSettings.GetActivePage: integer;
begin
  result := FIniFile.ReadInteger('UISettings', 'ActivePage', 0);
end;

function TSettings.GetConnType: integer;
begin
  result := FIniFile.ReadInteger('UserInfo', 'ConnType', 1);
end;

function TSettings.GetCoreIndex: integer;
begin
  result := finifile.ReadInteger('UISettings', 'CoreIndex', 0)
end;

procedure TSettings.GetFavoriteServerList(AStreamBuf: TStream);
begin
  AStreamBuf.Size := 0;
  FIniFile.ReadBinaryStream('ServerList', 'FavoriteServerList', AStreamBuf);
end;

procedure TSettings.GetFavServerList(ListBuf: TStrings);
begin
  FTempStream.Size := 0;
  FIniFile.ReadBinaryStream('ServerList', 'FavServerList', FTempStream);
  ListBuf.Text := FTempStream.DataString;
end;

function TSettings.GetJoinRing: boolean;
begin
  result := FIniFile.ReadBool('UISettings', 'JoinRing', true);
end;

function TSettings.GetLeaveMessage: string;
begin
  result := finifile.ReadString('UISettings', 'LeaveMessage',
    intfcommon.ProgramTitle+' '+intfcommon.GetAllVersion+' : '+intfcommon.authorurl
  );
end;

function TSettings.GetLocationTexts: string;
begin
  result := finifile.ReadString('UISettings', 'LocationTexts',
    'ѱ;korea;ڸ;ѹα;seoul;'
  );
end;

procedure TSettings.GetRoomFormPos(var X, Y: integer);
begin
  X := FIniFile.ReadInteger('UISettings', 'RoomFormPosX', 0);
  Y := FIniFile.ReadInteger('UISettings', 'RoomFormPosY', 0);
end;

function TSettings.GetShowingUserInOut: boolean;
begin
  result := finifile.ReadBool('UISettings', 'ShowingUserInOut', true);
end;

function TSettings.GetStayTop: boolean;
begin
  result := FIniFile.ReadBool('UISettings', 'StayTop', false);
end;

function TSettings.GetSVIndex: integer;
begin
  result := FIniFile.ReadInteger('UISettings', 'SVIndex', -1);
end;

function TSettings.GetUserName1: string;
begin
  result := FIniFile.ReadString('UserInfo', 'UserName', 'UserName1');
end;

procedure TSettings.GetVisitServerList(ListBuf: TStrings);
begin
  FTempStream.Size := 0;
  FIniFile.ReadBinaryStream('ServerList', 'VisitServerList', FTempStream);
  ListBuf.Text := FTempStream.DataString;
end;

procedure TSettings.SetActivePage(const Value: integer);
begin
  FIniFile.WriteInteger('UISettings', 'ActivePage', Value);
end;

procedure TSettings.SetConnType(const Value: integer);
begin
  FIniFile.WriteInteger('UserInfo', 'ConnType', Value);
end;

procedure TSettings.SetCoreIndex(const Value: integer);
begin
  finifile.WriteInteger('UISettings', 'CoreIndex', value);
end;

procedure TSettings.SetFavServerList(ServerList: TStrings);
begin
  FTempStream.Size := 0;
  FTempStream.WriteString(ServerList.Text);
  FTempStream.Position := 0;
  FIniFile.WriteBinaryStream('ServerList', 'FavServerList', FTempStream);
end;

procedure TSettings.SetJoinRing(const Value: boolean);
begin
  FIniFile.WriteBool('UISettings', 'JoinRing', Value);
end;

procedure TSettings.SetLeaveMessage(const Value: string);
begin
  finifile.WriteString('UISettings', 'LeaveMessage', value);
end;

procedure TSettings.SetLocationTexts(const Value: string);
begin
  finifile.WriteString('UISettings', 'LocationTexts', Value);
end;

procedure TSettings.SetRoomFormPos(const X, Y: integer);
begin
  FIniFile.WriteInteger('UISettings', 'RoomFormPosX', X);
  FIniFile.WriteInteger('UISettings', 'RoomFormPosY', Y);
end;

procedure TSettings.SetShowingUserInOut(const Value: boolean);
begin
  finifile.WriteBool('UISettings', 'ShowingUserInOut', Value);
end;

procedure TSettings.SetStayTop(const Value: boolean);
begin
  FIniFile.WriteBool('UISettings', 'StayTop', Value);
end;

procedure TSettings.SetSVIndex(const Value: integer);
begin
  FIniFile.WriteInteger('UISettings', 'SVIndex', Value);
end;

procedure TSettings.SetUserName(const Value: string);
begin
  FIniFile.WriteString('UserInfo', 'UserName', Value);
end;

procedure TSettings.SetVisitServerList(ServerList: TStrings);
begin
  FTempStream.Size := 0;
  FTempStream.WriteString(ServerList.Text);
  FTempStream.Position := 0;
  FIniFile.WriteBinaryStream('ServerList', 'VisitServerList', FTempStream);
end;

initialization
  Settings := TSettings.Create;

finalization
  Settings.Free;

end.
