unit uUserInterface;

interface

uses uCore, uIntfCommon, UIActions;

type
  TAppCallbacks = record
    gameCallback: function(game: PAnsiChar; player: integer; numplayers: integer): integer; stdcall;
    chatReceivedCallback: procedure(nick: PAnsiChar; text: PAnsiChar); stdcall;
    clientDroppedCallback: procedure(nick: PAnsiChar; playernb: integer); stdcall;
    moreInfosCallback: procedure(gamename: PAnsiChar); stdcall;
  end;

  TUserInterface = class
  private
    FAppCallbacks: TAppCallbacks;
  public
    constructor Create;
    destructor Destroy; override;
    procedure ShowMainForm;
    procedure SetAppName(const AppName: string);
    procedure SetGameList(const AGameList: PAnsiChar);
    procedure SetAppCallbacks(AppCallbacks: TAppCallbacks);
    procedure AddCoreIndex(ACoreName: string; ACoreIndex: integer);
    procedure SetCoreSelectProc(ACoreSelectProc: TCoreSelectProc);
    { calbacks }
    procedure GameCreated(Sender: TObject; AGameTitle: string; ID: integer; App: string; Owner: string);
    procedure GameClosed(Sender: TObject; ID: integer);
    procedure MyGameJoined(Sender: TObject);
    procedure MyGameLeft(Sender: TObject);
    procedure GameDropped(Sender: TObject; TerminateGame: boolean; APlayerName: string; APlayerNumber: integer);
    procedure GameStarted(Sender: TObject; GameTitle: string; Player, Players: integer);
    procedure GameStopped(Sender: TObject);
    procedure LoginRespond(Sender: TObject; LoginStatus: TLoginStatus; Reason: string);
    procedure Chated(Sender: TObject; Name, Message: string);
    procedure GameChated(Sender: TObject; Name, Message: string);
    procedure ServerMessaged(Sender: TObject; AMessage: string);
    procedure UserJoined(Sender: TObject; Name: string; Ping, ID, Conn: integer);
    procedure UserLeft(Sender: TObject; Name: string; LeaveMessage: string; AUserID: integer);
    procedure GameAdd(Sender: TObject; Title: string; ID: integer; AppName, Owner, Users: string; Status: integer);
    procedure UserAdd(Sender: TObject; Name: string; Ping, Status, ID, Conn: integer);
    procedure GameStatusChanged(Sender: TObject; ID: integer; Status: integer; NumPlayers: integer; MaxPlayers: integer);
    procedure PlayerJoined(Sender: TObject; UserName: string; ID: integer);
    procedure PlayerLeft(Sender: TObject; UserName: string; ID: integer);
    procedure PlayerAdd(Sender: TObject; ID: integer);
    procedure NetSyncWait(Sender: TObject; DelayedSec: integer; LimitSec: integer);
    procedure KickedOut(Sender: TObject);
  end;

var
  ui: tuserinterface;

implementation

uses forms, graphics, sysutils, log,
  uSelectServerDialog, uInServerDialog, uAboutBox, uAddrInputDialog,
  uAddServerDialog, uRoomForm, uSettings, uoptiondialog;

{ TIntfCaller }

constructor TUserInterface.Create;
begin
  application.HintPause := 3000;
end;

destructor TUserInterface.Destroy;
begin
  inherited;
end;

procedure TUserInterface.SetCoreSelectProc(ACoreSelectProc: TCoreSelectProc);
begin
  intfcommon.CoreSelector.SetCoreSelectProc(acoreselectproc);
end;

procedure TUserInterface.AddCoreIndex(ACoreName: string; ACoreIndex: integer);
begin
  intfcommon.CoreSelector.AddCoreIndex(acorename, acoreindex);
end;

procedure TUserInterface.ShowMainForm;
begin
  intfcommon.CoreSelector.InitializeCore;
	SelectServerDialog := TSelectServerDialog.Create(nil);
	InServerDialog := TInServerDialog.Create(nil);
	AboutBox := TAboutBox.Create(nil);
	AddrInputDialog := TAddrInputDialog.Create(nil);
	AddServerDialog := TAddServerDialog.Create(nil);
	RoomForm := TRoomForm.Create(nil);
  optiondialog := toptiondialog.Create(nil);
  //
  SelectServerDialog.ShowModal;
  //
  optiondialog.Free;
	RoomForm.Free;
  AddServerDialog.Free;
	AddrInputDialog.Free;
  AboutBox.Free;
	InServerDialog.Free;
  SelectServerDialog.Free;
end;

procedure TUserInterface.SetGameList(const AGameList: PAnsiChar);
var
  i,j:integer;
  s: array[0..99] of ansichar;
begin
  intfcommon.gamelist.Clear;
  i:=0;
  repeat
    j:=0;
    while agamelist[i] <> #0 do
    begin
      s[j] := agamelist[i];
      inc(i); inc(j);
    end;
    s[j] := #0;
    if strlen(s) <> 0 then intfcommon.gamelist.Add(s);
    inc(i);
  until agamelist[i] = #0;
end;

procedure TUserInterface.SetAppName(const AppName: string);
begin
  intfcommon.AppName := appname;
end;

procedure TUserInterface.ServerMessaged(Sender: TObject; AMessage: string);
var ac: tchatup;
begin
  ac := tchatup.Create;
  ac.Message := amessage;
  ac.Color := clgreen;
  actionmanager.PerformAction(ac);
end;

procedure TUserInterface.SetAppCallbacks(AppCallbacks: TAppCallbacks);
begin
  FAppCallbacks := AppCallbacks;
end;

procedure TUserInterface.LoginRespond(Sender: TObject; LoginStatus: TLoginStatus; Reason: string);
var ac: tconnstat;
begin
  trace;
  ac := tconnstat.Create;
  case LoginStatus of
    Full: begin
      ac.Msg := ' .';
      debug(ac.Msg);
      ac.Color := clRed;
      actionmanager.PerformAction(ac);
    end;
    Success: begin
      ac.Msg := ' .';
      debug(ac.Msg);
      ac.Color := clBlue;
      actionmanager.PerformAction(ac);
      actionmanager.PerformAction(tenterserver.Create);
    end;
    ucore.Failed: begin
      ac.Msg := ' : '+reason;
      debug(ac.Msg);
      ac.Color := clred;
      actionmanager.PerformAction(ac);
    end;
    Rejected: begin
      ac.Msg := ' ź: '+reason;
      debug(ac.Msg);
      ac.Color := clred;
      actionmanager.PerformAction(ac);
    end;
  end;
end;

procedure TUserInterface.GameClosed(Sender: TObject; ID: integer);
//  ϳ 
var
  ac: tremovegame;
begin
  ac := tremovegame.Create;
  ac.GameID := id;
  actionmanager.PerformAction(ac);
end;

procedure TUserInterface.GameCreated(Sender: TObject; AGameTitle: string; ID: integer; App,
  Owner: string);
// ϳ 
var
  ac: tsetgame;
begin
  ac := tsetgame.Create;
  ac.title := agametitle;
  ac.id := id;
  ac.app := app;
  ac.owner := owner;
  ac.users := '1/2';
  ac.status := 0;
  actionmanager.PerformAction(ac);
end;

procedure TUserInterface.GameDropped(Sender: TObject;  TerminateGame: boolean; APlayerName: string;
  APlayerNumber: integer);
var
  ac: tgamechatup;
begin
  ac := tgamechatup.Create;
  ac.Message := '<'+APlayerName+'>  ߽ϴ.';
  ac.Color := clgreen;
  ActionManager.PerformAction(ac);
  if terminategame then
    fappcallbacks.clientDroppedCallback(pansichar(ansistring(aplayername)), aplayernumber);
end;

procedure TUserInterface.MyGameLeft(Sender: TObject);
//   ų ٸ濡 
begin
  actionmanager.PerformAction(texitgame.Create);
end;

procedure TUserInterface.GameStarted(Sender: TObject; GameTitle: string; Player,
  Players: integer);
begin
  actionmanager.PerformAction(tstartgame.Create);
  if assigned(fappcallbacks.gameCallback) then
    fappcallbacks.gamecallback(
      pansichar(ansistring(roomform.ingametitleedit.text)),
      player,
      players
    );
end;

procedure TUserInterface.GameStopped(Sender: TObject);
begin
  ActionManager.PerformAction(tstopgame.Create);
end;

procedure TUserInterface.KickedOut(Sender: TObject);
var ac: tchatup;
begin
  ac := tchatup.Create;
  ac.Message := '< Ǿϴ>';
  ac.Color := clgreen;
  ActionManager.PerformAction(ac);
  ActionManager.PerformAction(texitgame.Create);
end;

procedure TUserInterface.Chated(Sender: TObject; Name, Message: string);
var ac: tchatup;
begin
  ac := tchatup.Create;
  ac.Message := '<'+name+'> '+message;
  ac.Color := clblack;
  ActionManager.PerformAction(ac);
end;

procedure TUserInterface.GameChated(Sender: TObject; Name, Message: string);
var ac: tgamechatup;
begin
  if assigned(fappcallbacks.chatReceivedCallback) then fappcallbacks.chatReceivedCallback
  (
    pansichar(ansistring(name)),
    pansichar(ansistring(message))
  );
  ac := tgamechatup.Create;
  if name <> 'Server' then begin
    ac.Message := '<'+name+'> '+message;
    ac.Color := clblack;
  end else begin
    ac.Message := message;
    ac.Color := clgreen;
  end;
  ActionManager.PerformAction(ac);
end;

procedure TUserInterface.UserJoined(Sender: TObject; Name: string; Ping, ID, Conn: integer);
//  ϳ  
var
  ac: tsetuser;
  ac2: tchatup;
begin
	ac := Tsetuser.Create;
	ac.name := name;
	ac.ping := ping;
	ac.status := 1;
	ac.id := id;
	ac.conn := conn;
  ActionManager.PerformAction(ac);
  if intfcommon.ShowingUserInOut then begin
    ac2 := tchatup.Create;
    ac2.Message := ': '+name;
    ac2.Color := clMedGray;
    ActionManager.PerformAction(ac2);
  end;
end;

procedure TUserInterface.GameAdd(Sender: TObject; Title: string; ID: integer; AppName, Owner, Users: string; Status: integer);
//      
var
  ac: tsetgame;
begin
  ac := tsetgame.Create;
	ac.title := title;
	ac.id := id;
	ac.app := appname;
	ac.owner := owner;
	ac.users := users;
	ac.status := status;
  ActionManager.PerformAction(ac);
end;

procedure TUserInterface.UserAdd(Sender: TObject; Name: string; Ping, Status, ID, Conn: integer);
//   ߰Ǵ  
var
  ac: tsetuser;
begin
  ac := tsetuser.Create;
	ac.name := name;
	ac.ping := ping;
	ac.status := status;
	ac.id := id;
	ac.conn := conn;
  ActionManager.PerformAction(ac);
end;

procedure TUserInterface.UserLeft(Sender: TObject; Name: string; LeaveMessage: string; AUserID: integer);
//  ϳ  
var
  ac: tremoveuser;
  ac2: tchatup;
begin
  ac := tremoveuser.Create;
  ac.UserID := auserid;
  ActionManager.PerformAction(ac);
  if intfcommon.ShowingUserInOut then begin
    ac2 := tchatup.Create;
    ac2.Message := ': '+name+'('+leavemessage+')';
    ac2.Color := clmedgray;
    ActionManager.PerformAction(ac2);
  end;
end;

procedure TUserInterface.GameStatusChanged(Sender: TObject; ID: integer; Status: integer; NumPlayers: integer; MaxPlayers: integer);
var
  ac: tsetgame;
begin
  ac := tsetgame.Create;
	ac.id := id;
	ac.users := inttostr(numplayers) + '/' + inttostr(maxplayers);
	ac.status := status;
  ActionManager.PerformAction(ac);
end;

procedure TUserInterface.MyGameJoined(Sender: TObject);
//  濡  Ǵ 
begin
  ActionManager.PerformAction(tentergame.Create);
end;

procedure TUserInterface.PlayerJoined(Sender: TObject; UserName: string; ID: integer);
var
  ac: tsetplayer;
  ac2: tgamechatup;
begin
  ac := tsetplayer.Create;
  ac.UserID := id;
  ActionManager.PerformAction(ac);
  ac2 := tgamechatup.Create;
  ac2.Message := '<'+username+'> 濡 ̽ϴ.';
  ac2.Color := clgreen;
  ActionManager.PerformAction(ac2);
end;

procedure TUserInterface.PlayerLeft(Sender: TObject; UserName: string; ID: integer);
var
  ac: tremoveplayer;
  ac2: tgamechatup;
begin
  ac := tremoveplayer.Create;
  ac.UserID := id;
  ActionManager.PerformAction(ac);
  ac2 := tgamechatup.Create;
  ac2.Message := '<'+username+'>  ϴ.';
  ac2.Color := clgreen;
  ActionManager.PerformAction(ac2);
end;

procedure TUserInterface.PlayerAdd(Sender: TObject; ID: integer);
// PlayerJoined   ƾ
var
  ac: tsetplayer;
begin
  ac := tsetplayer.Create;
  ac.UserID := id;
  ActionManager.PerformAction(ac);
end;

procedure TUserInterface.NetSyncWait(Sender: TObject; DelayedSec: integer; LimitSec: integer);
var
  ac: tgamechatup;
begin
  ac := tgamechatup.Create;
  ac.Message := ' ٸϴ: '+inttostr(delayedsec)+'/'+inttostr(limitsec)+'';
  ac.Color := clblack;
  ActionManager.PerformAction(ac);
end;

(*
void __fastcall UserStatus(int Ping, int Status, int Id, int Conn)
{
	DBG("\n");
	TUserData *UserData = new TUserData(); // ۵
	UserData->ping = Ping;
	UserData->status = Status;
	UserData->id = Id;
	UserData->conn = Conn;
	InterfaceManager->PerformAction(aiSetUser, UserData);
}
*)

end.
