unit OpenKCore;

interface

uses Classes, uCore;

type

  TListener = class;

  TOpenKCore = class(TInterfacedObject, ICore)
  private
    hdll: thandle;
    Listener: TListener;
    procedure SetCallbacks;
    procedure LoadFunctions;
  public
    constructor Create;
    destructor Destroy; override;
    { methods for ui }
    function CheckPing(AAddress: string): integer;
    procedure Login(AAppName, AUserName, AAddress: string; ConnectionType: Byte);
    procedure Logout(LeaveMessage: string);
    procedure CreateGame(AGameTitle: string);
    procedure JoinGame(AGameID: integer);
    procedure LeaveGame;
    procedure StartGame;
    procedure DropGame;
    procedure SendChat(aMessage: string);
    procedure SendGameChat(AMessage: string);
    procedure KickPlayer(AUserID: integer);
    { methods for app }
    procedure TerminateGame;
    procedure Chat(AMessage: string);
    function SyncData(AValues: PAnsiChar; ASize: integer): integer;
    { Callbacks }
    procedure SetGameCreated(ACallback: TGameCreated);
    procedure SetGameClosed(ACallback: TGameClosed);
    procedure SetMyGameJoined(ACallback: TNotifyEvent);
    procedure SetMyGameLeft(ACallback: TNotifyEvent);
    procedure SetGameStarted(ACallback: TGameStarted);
    procedure SetGameStopped(ACallback: TNotifyEvent);
    procedure SetGameDropped(ACallback: TGameDropped);
    procedure SetChated(ACallback: TChated);
    procedure SetGameChated(ACallback: TChated);
    procedure SetLoginRespond(ACallback: TLoginRespond);
    procedure SetServerMessaged(ACallback: TServerMessaged);
    procedure SetUserJoined(ACallback: TUserJoined);
    procedure SetUserLeft(ACallback: TUserLeft);
    procedure SetUserAdd(ACallback: TUserAdd);
    procedure SetGameAdd(ACallback: TGameAdd);
    procedure SetGameStatusChanged(ACallback: TGameStatusChanged);
    procedure SetPlayerJoined(ACallback: TPlayerInOut);
    procedure SetPlayerLeft(ACallback: TPlayerInOut);
    procedure SetPlayerAdd(ACallback: TPlayerAdd);
    procedure SetNetSyncWait(ACallback: TNetSyncWait);
    procedure SetKickedOut(ACallback: TNotifyEvent);
    { other }
    procedure ProcessInstruction;
    procedure BreakServer;
  end;

  TListener = class(TThread)
  protected
    procedure Execute; override;
  public
    constructor Create;
  end;

implementation

uses kaillerautils, log, windows, forms;

type
  func_kaillera_ping_server = function(host: pansichar; port: integer; limit: integer = 1000): integer; stdcall;
  func_kaillera_step = procedure; stdcall;

  func_kaillera_core_initialize = function(port: integer; appname: pansichar; username: pansichar; connection_setting: shortint): boolean; stdcall;
  func_kaillera_core_connect = function(ip: pansichar; port: integer = 27888): boolean; stdcall;
  func_kaillera_disconnect = function(quitmsg: pansichar): boolean; stdcall;
  func_kaillera_core_cleanup = function(): boolean; stdcall;
  func_kaillera_core_get_port = function(): integer; stdcall;
  func_kaillera_chat_send = procedure(text: pansichar); stdcall;
  func_kaillera_game_chat_send = procedure(text: pansichar); stdcall;
  func_kaillera_kick_user  = procedure(id: word); stdcall;
  func_kaillera_join_game = procedure(id: word); stdcall;
  func_kaillera_create_game = procedure(name: pansichar); stdcall;
  func_kaillera_leave_game  = procedure(); stdcall;
  func_kaillera_start_game = procedure(); stdcall;
  func_kaillera_game_drop = procedure(); stdcall;
  func_kaillera_end_game = procedure(); stdcall;
  func_kaillera_modify_play_values  = function(values: pansichar; size: integer): integer; stdcall;
  func_kaillera_is_connected = function: boolean; stdcall;

  pkcore_callbacks = ^kcore_callbacks;
  kcore_callbacks = record
    kaillera_user_add_callback        : procedure(name: pansichar; ping, status: integer; id: word; conn: shortint); stdcall;
    kaillera_game_add_callback        : procedure(gname: pansichar; id: word; emulator, owner, users: pansichar; status: shortint); stdcall;
    kaillera_chat_callback            : procedure(name, msg: pansichar); stdcall;
    kaillera_game_chat_callback       : procedure(name, msg: pansichar); stdcall;
    kaillera_motd_callback            : procedure(name, msg: pansichar); stdcall;
    kaillera_user_join_callback       : procedure(name: pansichar; ping: integer; id: word; conn: shortint); stdcall;
    kaillera_user_leave_callback      : procedure(name: pansichar; quitmsg: pansichar; id: word); stdcall;
    kaillera_game_create_callback     : procedure(gname: pansichar; id: longword; emulator, owner: pansichar); stdcall;
    kaillera_game_close_callback      : procedure(id: longword); stdcall;
    kaillera_user_game_create_callback: procedure(); stdcall;
    kaillera_game_status_change_callback: procedure(id: longword; status: shortint; players, maxplayers: integer); stdcall;
    kaillera_user_game_closed_callback: procedure(); stdcall;
    kaillera_user_game_joined_callback: procedure(); stdcall;
    kaillera_player_add_callback: procedure(name: pansichar; ping: integer; id: word; conn: shortint); stdcall;
    kaillera_player_joined_callback: procedure(username: pansichar; ping: integer; uid: word; connset: shortint); stdcall;
    kaillera_player_left_callback: procedure(user: pansichar; id: word); stdcall;
    kaillera_user_kicked_callback: procedure(); stdcall;
    kaillera_login_stat_callback: procedure(lsmsg: pansichar); stdcall;
    kaillera_player_dropped_callback: procedure(user: pansichar; gdpl: integer); stdcall;
    kaillera_game_callback: procedure(game: pansichar; player, players: shortint); stdcall;
    kaillera_game_netsync_wait_callback: procedure(tx: integer); stdcall;
    kaillera_end_game_callback: procedure(); stdcall;
  end;

  func_kaillera_set_callbacks = procedure(callbacks: pkcore_callbacks); stdcall;

var
  { dll funcs }
  kaillera_ping_server : func_kaillera_ping_server ;
  kaillera_step : func_kaillera_step ;

  kaillera_core_initialize : func_kaillera_core_initialize ;
  kaillera_core_connect : func_kaillera_core_connect ;
  kaillera_disconnect : func_kaillera_disconnect ;
  kaillera_core_cleanup : func_kaillera_core_cleanup ;
  kaillera_core_get_port : func_kaillera_core_get_port ;
  kaillera_chat_send : func_kaillera_chat_send ;
  kaillera_game_chat_send : func_kaillera_game_chat_send ;
  kaillera_kick_user  : func_kaillera_kick_user  ;
  kaillera_join_game : func_kaillera_join_game ;
  kaillera_create_game : func_kaillera_create_game ;
  kaillera_leave_game  : func_kaillera_leave_game  ;
  kaillera_start_game : func_kaillera_start_game ;
  kaillera_game_drop : func_kaillera_game_drop ;
  kaillera_end_game : func_kaillera_end_game ;
  kaillera_modify_play_values  : func_kaillera_modify_play_values  ;
  kaillera_is_connected: func_kaillera_is_connected;

  kaillera_set_callbacks : func_kaillera_set_callbacks;

  { events }
  GameCreated: TGameCreated;
  GameClosed: TGameClosed;
  MyGameJoined: TNotifyEvent;
  MyGameLeft: TNotifyEvent;
  GameStarted: TGameStarted;
  GameStopped: TNotifyEvent;
  GameDropped: TGameDropped;
  Chated: TChated;
  GameChated: TChated;
  LoginRespond: TLoginRespond;
  ServerMessaged: TServerMessaged;
  UserJoined: TUserJoined;
  UserLeft: TUserLeft;
  UserAdd: TUserAdd;
  GameAdd: TGameAdd;
  GameStatusChanged: TGameStatusChanged;
  PlayerJoined: TPlayerInOut;
  PlayerLeft: TPlayerInOut;
  PlayerAdd: TPlayerAdd;
  NetSyncWait: TNetSyncWait;
  KickedOut: TNotifyEvent;

procedure kaillera_user_add_callback (name: pansichar; ping, status: integer; id: word; conn: shortint); stdcall;
begin
  trace;
  useradd(nil, name, ping, status, id, conn);
end;

procedure kaillera_game_add_callback (gname: pansichar; id: word; emulator, owner, users: pansichar; status: shortint); stdcall;
begin
  trace;
  gameadd(nil, gname, id, emulator, owner, users, status);
end;

procedure kaillera_chat_callback (name, msg: pansichar); stdcall;
begin
  trace;
  Chated(nil, name, msg);
end;

procedure kaillera_game_chat_callback (name, msg: pansichar); stdcall;
begin
  trace;
  gamechated(nil, name, msg);
end;

procedure kaillera_motd_callback (name, msg: pansichar); stdcall;
begin
  trace;
  servermessaged(nil, msg);
end;

procedure kaillera_user_join_callback (name: pansichar; ping: integer; id: word; conn: shortint); stdcall;
begin
  trace;
  userjoined(nil, name, ping, id, conn);
end;

procedure kaillera_user_leave_callback (name: pansichar; quitmsg: pansichar; id: word); stdcall;
begin
  trace;
  userleft(nil, name, quitmsg, id);
end;

procedure kaillera_game_create_callback (gname: pansichar; id: longword; emulator, owner: pansichar); stdcall;
begin
  trace;
  gamecreated(nil, gname, id, emulator, owner);
end;

procedure kaillera_game_close_callback (id: longword); stdcall;
begin
  trace;
  gameclosed(nil, id);
end;

procedure kaillera_user_game_create_callback(); stdcall;
begin
  trace;
  mygamejoined(nil);
end;

procedure kaillera_game_status_change_callback(id: longword; status: shortint; players, maxplayers: integer); stdcall;
begin
  trace;
  gamestatuschanged(nil, id, status, players, maxplayers);
end;

procedure kaillera_user_game_closed_callback(); stdcall;
begin
  trace;
  mygameleft(nil);
end;

procedure kaillera_user_game_joined_callback(); stdcall;
begin
  trace;
  mygamejoined(nil);
end;

procedure kaillera_player_add_callback(name: pansichar; ping: integer; id: word; conn: shortint); stdcall;
begin
  trace;
  playeradd(nil, id);
end;

procedure kaillera_player_joined_callback(username: pansichar; ping: integer; uid: word; connset: shortint); stdcall;
begin
  trace;
  playerjoined(nil, username, uid);
end;

procedure kaillera_player_left_callback(user: pansichar; id: word); stdcall;
begin
  trace;
  playerleft(nil, user, id);
end;

procedure kaillera_user_kicked_callback(); stdcall;
begin
  trace;
  kickedout(nil);
end;

procedure kaillera_login_stat_callback(lsmsg: pansichar); stdcall;
begin
  trace;
end;

procedure kaillera_player_dropped_callback(user: pansichar; gdpl: integer); stdcall;
begin
  trace;
  gamedropped(nil, true, user, gdpl);
end;

procedure kaillera_game_callback(game: pansichar; player, players: shortint); stdcall;
begin
  trace;
  gamestarted(nil, game, player, players);
end;

procedure kaillera_game_netsync_wait_callback(tx: integer); stdcall;
begin
  trace;
  netsyncwait(nil, tx, 60);
end;

procedure kaillera_end_game_callback(); stdcall;
begin
  trace;
  gamestopped(nil);
end;

{ TOpenKCore }

procedure TOpenKCore.BreakServer;
begin

end;

procedure TOpenKCore.Chat(AMessage: string);
begin
  kaillera_game_chat_send(pansichar(ansistring(amessage)));
end;

function TOpenKCore.CheckPing(AAddress: string): integer;
var host: string; port: word;
begin
  gethostport(host, port, aaddress);
  result := kaillera_ping_server(pansichar(ansistring(host)), port);
end;

constructor TOpenKCore.Create;
begin
  loadfunctions;
  setcallbacks;
  listener := tlistener.Create;
end;

procedure TOpenKCore.CreateGame(AGameTitle: string);
begin
  kaillera_create_game(pansichar(ansistring(agametitle)));
end;

destructor TOpenKCore.Destroy;
begin
  listener.Terminate;
  freelibrary(hdll);
  inherited;
end;

procedure TOpenKCore.DropGame;
begin
  kaillera_game_drop;
end;

procedure TOpenKCore.JoinGame(AGameID: integer);
begin
  kaillera_join_game(agameid);
end;

procedure TOpenKCore.KickPlayer(AUserID: integer);
begin
  kaillera_kick_user(auserid);
end;

procedure TOpenKCore.LeaveGame;
begin
  kaillera_leave_game;
end;

procedure TOpenKCore.LoadFunctions;
begin
  hdll := loadlibrary('open_kcore.dll');
  if hdll <> 0 then
  begin
    try
      kaillera_ping_server := GetProcAddress(hdll, 'kaillera_ping_server');
      kaillera_step := GetProcAddress(hdll, 'kaillera_step');
      kaillera_core_initialize := GetProcAddress(hdll, 'kaillera_core_initialize');
      kaillera_core_connect := GetProcAddress(hdll, 'kaillera_core_connect');
      kaillera_disconnect := GetProcAddress(hdll, 'kaillera_disconnect');
      kaillera_core_cleanup := GetProcAddress(hdll, 'kaillera_core_cleanup');
      kaillera_core_get_port := GetProcAddress(hdll, 'kaillera_core_get_port');
      kaillera_chat_send := GetProcAddress(hdll, 'kaillera_chat_send');
      kaillera_game_chat_send := GetProcAddress(hdll, 'kaillera_game_chat_send');
      kaillera_kick_user  := GetProcAddress(hdll, 'kaillera_kick_user ');
      kaillera_join_game := GetProcAddress(hdll, 'kaillera_join_game');
      kaillera_create_game := GetProcAddress(hdll, 'kaillera_create_game');
      kaillera_leave_game  := GetProcAddress(hdll, 'kaillera_leave_game');
      kaillera_start_game := GetProcAddress(hdll, 'kaillera_start_game');
      kaillera_game_drop := GetProcAddress(hdll, 'kaillera_game_drop');
      kaillera_end_game := GetProcAddress(hdll, 'kaillera_end_game');
      kaillera_modify_play_values  := GetProcAddress(hdll, 'kaillera_modify_play_values');
      kaillera_is_connected := GetProcAddress(hdll, 'kaillera_is_connected');
      kaillera_set_callbacks := getprocaddress(hdll, 'kaillera_set_callbacks');
    except
      writeln('error on functions loading');
    end;
  end
  else
    writeln('couldn''t load library');
end;

procedure TOpenKCore.Login(AAppName, AUserName, AAddress: string;
  ConnectionType: Byte);
var
  host: string; port: word;
begin
  if not gethostport(host, port, aaddress) then exit;
  writeln('login to ',host,':',port);
  if not kaillera_core_initialize(0, pansichar(ansistring(aappname)), pansichar(ansistring(ausername)), connectiontype) then
  begin
    debug('couldn''t init');
    exit;
  end;
  if kaillera_core_connect(pansichar(ansistring(host)), port) then
    loginrespond(self, success)
  else
    loginrespond(self, ucore.failed);
end;

procedure TOpenKCore.Logout(LeaveMessage: string);
begin
  kaillera_disconnect(pansichar(ansistring(leavemessage)));
end;

procedure TOpenKCore.ProcessInstruction;
begin
  kaillera_step();
end;

procedure TOpenKCore.SendChat(aMessage: string);
begin
  kaillera_chat_send(pansichar(ansistring(amessage)));
end;

procedure TOpenKCore.SendGameChat(AMessage: string);
begin
  kaillera_game_chat_send(pansichar(ansistring(amessage)));
end;

procedure TOpenKCore.SetCallbacks;
var
  kc: kcore_callbacks;
begin
  kc.kaillera_user_add_callback :=                 kaillera_user_add_callback;
  kc.kaillera_game_add_callback :=                 kaillera_game_add_callback;
  kc.kaillera_chat_callback :=                     kaillera_chat_callback;
  kc.kaillera_game_chat_callback :=                kaillera_game_chat_callback;
  kc.kaillera_motd_callback :=                     kaillera_motd_callback;
  kc.kaillera_user_join_callback :=                kaillera_user_join_callback;
  kc.kaillera_user_leave_callback :=               kaillera_user_leave_callback;
  kc.kaillera_game_create_callback :=              kaillera_game_create_callback;
  kc.kaillera_game_close_callback :=               kaillera_game_close_callback;
  kc.kaillera_user_game_create_callback :=         kaillera_user_game_create_callback;
  kc.kaillera_game_status_change_callback :=       kaillera_game_status_change_callback;
  kc.kaillera_user_game_closed_callback :=         kaillera_user_game_closed_callback;
  kc.kaillera_user_game_joined_callback :=         kaillera_user_game_joined_callback;
  kc.kaillera_player_add_callback :=               kaillera_player_add_callback;
  kc.kaillera_player_joined_callback :=            kaillera_player_joined_callback;
  kc.kaillera_player_left_callback :=              kaillera_player_left_callback;
  kc.kaillera_user_kicked_callback :=              kaillera_user_kicked_callback;
  kc.kaillera_login_stat_callback :=               kaillera_login_stat_callback;
  kc.kaillera_player_dropped_callback :=           kaillera_player_dropped_callback;
  kc.kaillera_game_callback :=                     kaillera_game_callback;
  kc.kaillera_game_netsync_wait_callback :=        kaillera_game_netsync_wait_callback;
  kc.kaillera_end_game_callback :=                 kaillera_end_game_callback;
  kaillera_set_callbacks(@kc);
end;

procedure TOpenKCore.SetChated(ACallback: TChated);
begin
  chated := acallback;
end;

procedure TOpenKCore.SetGameAdd(ACallback: TGameAdd);
begin
  gameadd := acallback;
end;

procedure TOpenKCore.SetGameChated(ACallback: TChated);
begin
  gamechated := acallback;
end;

procedure TOpenKCore.SetGameClosed(ACallback: TGameClosed);
begin
  gameclosed := acallback;
end;

procedure TOpenKCore.SetGameCreated(ACallback: TGameCreated);
begin
  gamecreated := acallback;
end;

procedure TOpenKCore.SetGameDropped(ACallback: TGameDropped);
begin
  gamedropped := acallback;
end;

procedure TOpenKCore.SetGameStarted(ACallback: TGameStarted);
begin
  gamestarted := acallback;
end;

procedure TOpenKCore.SetGameStatusChanged(ACallback: TGameStatusChanged);
begin
  gamestatuschanged := acallback;
end;

procedure TOpenKCore.SetGameStopped(ACallback: TNotifyEvent);
begin
  gamestopped := acallback;
end;

procedure TOpenKCore.SetKickedOut(ACallback: TNotifyEvent);
begin
  kickedout := acallback;
end;

procedure TOpenKCore.SetLoginRespond(ACallback: TLoginRespond);
begin
  loginrespond := acallback;
end;

procedure TOpenKCore.SetMyGameJoined(ACallback: TNotifyEvent);
begin
  mygamejoined := acallback;
end;

procedure TOpenKCore.SetMyGameLeft(ACallback: TNotifyEvent);
begin
  mygameleft := acallback;
end;

procedure TOpenKCore.SetNetSyncWait(ACallback: TNetSyncWait);
begin
  netsyncwait := acallback;
end;

procedure TOpenKCore.SetPlayerAdd(ACallback: TPlayerAdd);
begin
  playeradd := acallback;
end;

procedure TOpenKCore.SetPlayerJoined(ACallback: TPlayerInOut);
begin
  playerjoined := acallback;
end;

procedure TOpenKCore.SetPlayerLeft(ACallback: TPlayerInOut);
begin
  playerleft := acallback;
end;

procedure TOpenKCore.SetServerMessaged(ACallback: TServerMessaged);
begin
  servermessaged := acallback;
end;

procedure TOpenKCore.SetUserAdd(ACallback: TUserAdd);
begin
  useradd := acallback;
end;

procedure TOpenKCore.SetUserJoined(ACallback: TUserJoined);
begin
  userjoined := acallback;
end;

procedure TOpenKCore.SetUserLeft(ACallback: TUserLeft);
begin
  userleft := acallback;
end;

procedure TOpenKCore.StartGame;
begin
  kaillera_start_game;
end;

function TOpenKCore.SyncData(AValues: PAnsiChar; ASize: integer): integer;
begin
  result := kaillera_modify_play_values(AValues, ASize);
end;

procedure TOpenKCore.TerminateGame;
begin
  kaillera_end_game;
end;

{ TListener }

constructor TListener.Create;
begin
  inherited create(false);
  freeonterminate := true;
end;

procedure TListener.Execute;
begin
  inherited;
  while not terminated do
  begin
    while kaillera_is_connected do
    begin
      kaillera_step;
    end;
    sleep(1);
  end;
end;

end.
