unit uCore;

interface

uses classes;

type
  TGameCreated = procedure(Sender: TObject; gname: string; id: integer; emulator: string; owner: string) of object;
  TGameClosed = procedure(Sender: TObject; ID: integer) of object;
  TGameStarted = procedure(Sender: TObject; GameTitle: string; Player, Players: integer) of object;
  TGameDropped = procedure(Sender: TObject;  TerminateGame: boolean; APlayerName: string; APlayerNumber: integer) of object;
  TLoginStatus = (Success, Full, Failed, Rejected);
  TLoginRespond = procedure(Sender: TObject; LoginStatus: TLoginStatus; Reason: string = '') of object;
  TChated = procedure(Sender: TObject; Name, Message: string) of object;
  TServerMessaged = procedure(Sender: TObject; Message: string) of object;
  TUserJoined = procedure(Sender: TObject; Name: string; Ping, ID, Conn: integer) of object;
  TUserLeft = procedure(Sender: TObject; Name: string; LeaveMessage: string; AUserID: integer) of object;
  TGameAdd = procedure(Sender: TObject; Title: string; ID: integer; AppName, Owner, Users: string; Status: integer) of object;
  TUserAdd = procedure(Sender: TObject; Name: string; Ping, Status, ID, Conn: integer) of object;
  TGameStatusChanged = procedure(Sender: TObject; ID: integer; Status: integer; NumPlayers: integer; MaxPlayers: integer) of object;
  TPlayerInOut = procedure(Sender: TObject; UserName: string; ID: integer) of object;
  TPlayerAdd = procedure(Sender: TObject; ID: integer) of object;
  TNetSyncWait = procedure(Sender: TObject; DelayedSec: integer; LimitSec: integer) of object;

  ICore = interface
    ['{5415F0D3-7DD4-4AB1-A5BB-23D5C703C5F4}']
    { methods for ui }
    function CheckPing(AAdress: string): integer;
    procedure Login(AAppName, AUserName, AAddress: string; ConnectionType: Byte);
    procedure Logout(LeaveMessage: string);
    procedure CreateGame(AGameTitle: string);
    procedure JoinGame(AGameID: integer);
    procedure LeaveGame;
    procedure StartGame;
    procedure DropGame;
    procedure SendChat(Message: string);
    procedure SendGameChat(AMessage: string);
    procedure KickPlayer(AUserID: integer);
    { methods for app }
    procedure TerminateGame;
    procedure Chat(AMessage: string);
    function SyncData(AValues: PAnsiChar; ASize: integer): integer;
    { Callbacks }
    procedure SetGameCreated(ACallback: TGameCreated);
    procedure SetGameClosed(ACallback: TGameClosed);
    procedure SetMyGameJoined(ACallback: TNotifyEvent);
    procedure SetMyGameLeft(ACallback: TNotifyEvent);
    procedure SetGameStarted(ACallback: TGameStarted);
    procedure SetGameStopped(ACallback: TNotifyEvent);
    procedure SetGameDropped(ACallback: TGameDropped);
    procedure SetChated(ACallback: TChated);
    procedure SetGameChated(ACallback: TChated);
    procedure SetLoginRespond(ACallback: TLoginRespond);
    procedure SetServerMessaged(ACallback: TServerMessaged);
    procedure SetUserJoined(ACallback: TUserJoined);
    procedure SetUserLeft(ACallback: TUserLeft);
    procedure SetUserAdd(ACallback: TUserAdd);
    procedure SetGameAdd(ACallback: TGameAdd);
    procedure SetGameStatusChanged(ACallback: TGameStatusChanged);
    procedure SetPlayerJoined(ACallback: TPlayerInOut);
    procedure SetPlayerLeft(ACallback: TPlayerInOut);
    procedure SetPlayerAdd(ACallback: TPlayerAdd);
    procedure SetNetSyncWait(ACallback: TNetSyncWait);
    procedure SetKickedOut(ACallback: TNotifyEvent);
    { other }
    procedure BreakServer;
  end;

var
  Core: ICore;

implementation

end.
