unit uKailleraClient;

{$IFDEF FPC}
{$mode delphi}
{$ENDIF}

interface

uses SysUtils, IdUDPClient, Classes, Contnrs, ExtCtrls,
  instructions2, kaillerautils, ucore;

type

  TListener = class;

  TKailleraClient = class(TInterfacedObject, ICore)
  private
    { common record }
    FMyName: string;
    FMyID: integer;
    FMyGameID: word;
    FIgnoreFrameCount: cardinal;
    FFrameNo: cardinal;
    FAllReady: boolean;
    FConnection: Byte;
    FPlayerNo: integer;
    FPlayers: integer;
    FLeaveGameRequested: boolean;
    FGameSyncing: boolean;
    FLoggedIn: boolean;
    FNotifyWindow: THandle;
    { member objects }
    FUDPClient: TIdUDPClient;
    FGameDataSendBuffer: TGameDataBuffer;
    FGameDataRecvBuffer: TGameDataBuffer;
    FGameDataRecvCache: TGameDataCache;
    FGameDataSendCache: TGameDataCache;
    FIncomingInstructions: TInstructionRecvQueue;
    FOutgoingInstructions: TInstructionSendQueue;
    FConnectionKeeper: TTimer;
    FListener: tlistener;
    { events }
    GameCreated: TGameCreated;
    GameClosed: TGameClosed;
    MyGameJoined: TNotifyEvent;
    MyGameLeft: TNotifyEvent;
    GameStarted: TGameStarted;
    GameStopped: TNotifyEvent;
    GameDropped: TGameDropped;
    Chated: TChated;
    GameChated: TChated;
    LoginRespond: TLoginRespond;
    ServerMessaged: TServerMessaged;
    UserJoined: TUserJoined;
    UserLeft: TUserLeft;
    UserAdd: TUserAdd;
    GameAdd: TGameAdd;
    GameStatusChanged: TGameStatusChanged;
    PlayerJoined: TPlayerInOut;
    PlayerLeft: TPlayerInOut;
    PlayerAdd: TPlayerAdd;
    NetSyncWait: TNetSyncWait;
    KickedOut: TNotifyEvent;
    { end events }
    procedure SendKeepAlive(Sender: TObject);
    procedure SendInstruction(AInstruction: PInstruction; FreeIns: boolean = true);
    function RecvInstruction: PInstruction;
    procedure DispatchInstruction(AInstruction: PInstruction);
    procedure CleanupSocket;
    function Connect(AAddress: string): boolean;
    procedure Disconnect;
  public
    constructor Create;
    destructor Destroy; override;
    { methods for ui }
    function CheckPing(AAdress: string): integer;
    procedure Login(AAppName, AUserName, AAddress: string; ConnectionType: Byte);
    procedure Logout(LeaveMessage: string);
    procedure CreateGame(AGameTitle: string);
    procedure JoinGame(AGameID: integer);
    procedure LeaveGame;
    procedure StartGame;
    procedure DropGame;
    procedure BreakServer;
    procedure SendChat(Message: string);
    procedure SendGameChat(AMessage: string);
    procedure KickPlayer(AUserID: integer);
    { methods for app }
    procedure TerminateGame;
    procedure Chat(AMessage: string);
    function SyncData(AValues: PAnsiChar; ASize: integer): integer;
    procedure SendData(const AValues; ASize: integer);
    procedure RecvData(out AValues; out ASize: integer);
    { other }
    procedure ProcessInstruction();
    procedure SetNotifyWindow(AWindow: THandle);
    { Callbacks }
    procedure SetGameCreated(ACallback: TGameCreated);
    procedure SetGameClosed(ACallback: TGameClosed);
    procedure SetMyGameJoined(ACallback: TNotifyEvent);
    procedure SetMyGameLeft(ACallback: TNotifyEvent);
    procedure SetGameStarted(ACallback: TGameStarted);
    procedure SetGameStopped(ACallback: TNotifyEvent);
    procedure SetGameDropped(ACallback: TGameDropped);
    procedure SetChated(ACallback: TChated);
    procedure SetGameChated(ACallback: TChated);
    procedure SetLoginRespond(ACallback: TLoginRespond);
    procedure SetServerMessaged(ACallback: TServerMessaged);
    procedure SetUserJoined(ACallback: TUserJoined);
    procedure SetUserLeft(ACallback: TUserLeft);
    procedure SetUserAdd(ACallback: TUserAdd);
    procedure SetGameAdd(ACallback: TGameAdd);
    procedure SetGameStatusChanged(ACallback: TGameStatusChanged);
    procedure SetPlayerJoined(ACallback: TPlayerInOut);
    procedure SetPlayerLeft(ACallback: TPlayerInOut);
    procedure SetPlayerAdd(ACallback: TPlayerAdd);
    procedure SetNetSyncWait(ACallback: TNetSyncWait);
    procedure SetKickedOut(ACallback: TNotifyEvent);
  end;

  TListener = class(TThread)
  private
    this: TKailleraClient;
  protected
    procedure Execute; override;
  public
    constructor Create(AKailleraClient: TKailleraClient);
  end;

implementation

uses strutils, windows, log, idglobal, idstack, messages, forms;

const
  MAX_PACKET_SIZE = 6000;
  MAX_DELAY = 1000;

{
  1,2,3
  2,3,4 *
  3,4,5 *
  4,5,6 *
  5,6,7
}

var
  RecvBuffer: tpacket;

procedure TKailleraClient.JoinGame(AGameID: integer);
begin
  fmygameid := agameid; // PJN  gameid  ޹ ϴ  ⿡
  sendinstruction(new(ins_pjn,init(fconnection, agameid)));
end;

procedure TKailleraClient.KickPlayer(AUserID: integer);
begin
  sendinstruction(new(ins_kck,init(auserid)));
end;

{ TKaiConnection }

procedure TKailleraClient.Login(AAppName, AUserName, AAddress: string; ConnectionType: Byte);
begin
  if floggedin then
  begin
    if assigned(loginrespond) then
      loginrespond(self, uCore.Failed, '̹  Ǿ ');
    exit;
  end;
  disconnect;
  if not connect(aaddress) then exit;
  FConnection := ConnectionType;
  FMyName := AuserName;
  FLoggedIn := false;

  writeln('α û');
  SendInstruction(new(ins_lrq,Init(ausername, aappname, connectiontype)));
end;

procedure TKailleraClient.SendChat(Message: string);
begin
  sendinstruction(new(ins_cht,init(message)));
end;

procedure TKailleraClient.SendData(const AValues; ASize: integer);
begin

end;

procedure TKailleraClient.SendGameChat(AMessage: string);
begin
  sendinstruction(new(ins_gch,init(amessage)));
end;

procedure TKailleraClient.BreakServer;
var buf:tbytes;
begin
  setlength(buf, 10000);
  fillchar(buf[0], 10000, $FF);
  fudpclient.SendBuffer(buf);
end;

constructor TKailleraClient.Create;
begin
  FUDPClient := TIdUDPClient.Create(nil);
//  fudpclient.TransparentProxy.Host := '221.241.160.111';
//  fudpclient.TransparentProxy.Port := 8080;

  FIncomingInstructions := TInstructionRecvQueue.Create;
  foutgoinginstructions := TInstructionSendQueue.Create;
  fgamedatasendbuffer := tgamedatabuffer.Create;
  fgamedatarecvbuffer := tgamedatabuffer.Create;
  FGameDataRecvCache := TGameDataCache.Create;
  fgamedatasendcache := tgamedatacache.Create;
  FConnectionKeeper := TTimer.Create(nil);
  FConnectionKeeper.Enabled := false;
  FConnectionKeeper.Interval := 30000;
  FConnectionKeeper.OnTimer := SendKeepAlive;
  FLeaveGameRequested := false;
  FListener := tlistener.Create(self);
end;

destructor TKailleraClient.Destroy;
begin
  FListener.free;
  FConnectionKeeper.Free;
  fgamedatasendbuffer.Free;
  fgamedatarecvbuffer.Free;
  fgamedatasendcache.Free;
  FGameDataRecvCache.Free;
  foutgoinginstructions.Free;
  FIncomingInstructions.Free;
  FUDPClient.Free;
  inherited;
end;

procedure TKailleraClient.CreateGame(AGameTitle: string);
begin
  SendInstruction(new(ins_gcr,init(agametitle)));
end;

procedure TKailleraClient.Logout(LeaveMessage: string);
begin
  sendinstruction(new(ins_ulv,Init(leavemessage)));
  disconnect;
end;

procedure TKailleraClient.ProcessInstruction();
var
  ins: pinstruction;
begin
  ins := RecvInstruction;
  if ins <> nil then DispatchInstruction(ins);
end;

function TKailleraClient.CheckPing(AAdress: string): integer;
var host: string; port: tport;
  recvstr: string;
  tick: cardinal;
begin
  result := -1;
  if not gethostport(host, port, aadress) then exit;
  writeln('check ping to ',host,':',port);
  recvstr := '';
  tick := gettickcount;
  try
    fudpclient.Send(host, port, 'PING'#0);
    recvstr := fudpclient.ReceiveString(MAX_DELAY);
  except
  end;
  if recvstr = 'PONG'#0 then
    result := gettickcount - tick;
end;

procedure TKailleraClient.RecvData(out AValues; out ASize: integer);
begin

end;

function TKailleraClient.RecvInstruction: PInstruction;
var
  size: integer;
begin
  size := 0;
  try
    size := FUDPClient.ReceiveBuffer(TIdBytes(RecvBuffer), 1000);
  except
    on E: EIdSocketError do
      if E.LastError = 10040 then writeln(E.Message);
  end;
  if size <> 0 then
  begin
    FIncomingInstructions.ExtractData(@RecvBuffer[0], size);
    result := PInstruction(FIncomingInstructions.Pop);
  end
  else result := nil;
end;

procedure TKailleraClient.Disconnect;
begin
  FConnectionKeeper.enabled := false;
  fudpclient.Disconnect;
  floggedin := false;
end;

procedure TKailleraClient.DispatchInstruction(AInstruction: PInstruction);
var i: integer;
begin
  writeln('disp id: ', ainstruction.GetIDString);
  case AInstruction.getID of
    PING: begin
      SendInstruction(new(INS_PONG,init));
    end;
    ULV: with ins_ulv(ainstruction)^ do begin
      if FMyID = UserId then disconnect;
      if assigned(userleft) then userleft(self, username, LeaveMessage, UserId);
    end;
    GCR: with ins_gcr(ainstruction)^ do
      if assigned(gamecreated) then
        gamecreated(self, GameName, GameId, AppName, UserName);
    GCL: with INS_GCL(AInstruction)^ do
    begin
      if assigned(GameClosed) then gameclosed(self, GameID);
      if FMyGameID = GameID then if assigned(mygameleft) then mygameleft(self);
    end;
    PJN: with INS_PJN(ainstruction)^ do
    begin
      writeln('player joined, id: ', UserID);
      if UserID = FMyID then if assigned(mygamejoined) then mygamejoined(self);
      if assigned(playerjoined) then playerjoined(self, playername, userid);
    end;
    PLV: with INS_PLV(AInstruction)^ do
    begin
      if UserID = FMyID then begin
        if not fleavegamerequested then if assigned(kickedout) then kickedout(self);
        if assigned(MyGameLeft) then MyGameLeft(self);
        fleavegamerequested := false;
      end;
      if assigned(playerleft) then playerleft(self, username, userid);
    end;
    PLS: with INS_PLS(ainstruction)^ do
    begin
      if assigned(mygamejoined) then mygamejoined(self);
      if assigned(playeradd) then
        for i:= 1 to playerscount do
        begin
          writeln(players[i].Name, ' ', players[i].ID);
          playeradd(self, players[i].ID);
        end;
    end;
    UJN: with ins_ujn(ainstruction)^ do
    begin
      if fmyid = -1 then fmyid := UserId;
      if assigned(userjoined) then userjoined(self, username, ping, userid, connectiontype);
    end;
    ULS: WITH INS_ULS(ainstruction)^ do
    begin
      writeln(format('uls, userscount: %d, gamescount: %d', [userscount, gamescount]));
      if not floggedin then
      begin
        floggedin := true;
        if assigned(loginrespond) then loginrespond(self, success, '');
        FConnectionKeeper.Enabled := true;
      end;
      if assigned(useradd) then
      begin
        for i:= 1 to userscount do
          with users[i] do useradd(self, name, ping, status, id, conn);
      end;
      if assigned(gameadd) then
      begin
        for i:= 1 to gamescount do
          with games[i] do gameadd(self, title, id, appname, owner, userscount, status);
      end;
    end;
    SVR: with INS_SVR(ainstruction)^ do
    begin
      if assigned(servermessaged) then servermessaged(self, Msg);
    end;
    CHT: with ins_cht(ainstruction)^ do
      if assigned(chated) then chated(self, UserName, Message);
    GCH: with INS_GCH(ainstruction)^ do
    begin
      if assigned(gamechated) then gamechated(self, username, message);
    end;
    GST: with INS_GST(ainstruction)^ do
      if assigned(GameStatusChanged) then GameStatusChanged(self, GameID, GameStatus, NumPlayers, maxPlayers);
    RJT: with INS_RJT(ainstruction)^ do
    begin
      if fmyid = UserId then disconnect;
      if assigned(loginrespond) then loginrespond(self, Rejected, Msg);
      writeln('login rejected: ', Msg);
    end;
    DRP: with ins_drp(ainstruction)^ do
    begin
      if assigned(gamedropped) then
        gamedropped(self, FGameSyncing, UserName, PlayerNumber);
    end;
    GSA: with ins_gsa(ainstruction)^ do
    begin
      fignoreframecount := fconnection * (TimeBand+1) - 1;
      fframeno := 0;
      fplayerno := PlayerNo;
      fplayers := MaxPlayers;
      fallready := false;
      FGameSyncing := true;
      fgamedatasendcache.Clear;
      fgamedatarecvcache.Clear;
      fgamedatasendbuffer.Clear;
      fgamedatarecvbuffer.Clear;
      fconnectionkeeper.enabled := false;
      writeln('<MY GAME STARTED, timeband: ', timeband, '>');
      if assigned(gamestarted) then gamestarted(self, '', PlayerNo, MaxPlayers);
      //  ε ..
      if assigned(gamestopped) then gamestopped(self);
      writeln('<GAME END>');
      fconnectionkeeper.enabled := true;
      fgamesyncing := false;
    end;
    RDY:
    begin
      fallready := true;
    end;
    gda: with ins_gda(ainstruction)^ do
    begin
      fgamedatarecvbuffer.Push(data, size);
      if FGameDataRecvCache.Find(data, size) = -1 then
      begin
        FGameDataRecvCache.Add(ainstruction);
        exit;
      end;
    end;
    CDA: with ins_cda(ainstruction)^ do
    begin
      with FGameDataRecvCache.Get(index)^ do
        fgamedatarecvbuffer.Push(data, size);
    end;
  end;
  dispose(AInstruction,done);
end;

procedure TKailleraClient.DropGame;
begin
  SendInstruction(new(ins_drp,init));
end;

procedure TKailleraClient.Chat(AMessage: string);
begin
  sendgamechat(amessage);
end;

procedure TKailleraClient.CleanupSocket;
begin
  fudpclient.Binding.Reset();
end;

function TKailleraClient.Connect(AAddress: string): boolean;
var
  recvstr: string;
  host: string; port: word;
begin
  result := false;
  {   Ȯ }
  if not gethostport(host, port, AAddress) then
  begin
    if assigned(loginrespond) then loginrespond(self, ucore.failed, 'ּ̻');
    exit;
  end;
  { ʱȭ }
  CleanupSocket;
  fmyid := -1;
  FIncomingInstructions.Clear;
  foutgoinginstructions.Clear;
  {  }
  try
    writeln('ӿû');
    try
      writeln('connection requesting to ', host, ' ', port);
      FUDPClient.Send(host, port, 'HELLO0.83'#0);
      recvstr := FUDPClient.ReceiveString(max_delay);
    except
      on e: eidsocketerror do begin
        loginrespond(self, ucore.Failed, '');
        exit;
      end;
    end;
    writeln('"'+recvstr+'"');
    if not ansicontainsstr(recvstr, 'HELLOD00D') then
    begin
      if ansicontainsstr(recvstr, 'TOO') then begin
        if assigned(loginrespond) then
          loginrespond(self, ucore.Full, '');
      end
      else loginrespond(self, ucore.Failed, '');
      exit;
    end;

    recvstr := stringreplace(recvstr, 'HELLOD00D', '', []);
    port := strtoint(recvstr);
    writeln('port allowed : ', port);
    fudpclient.Host := host;
    fudpclient.Port := port;
    fudpclient.Connect; // ӵ  
    result := true;

  except
    if assigned(loginrespond) then loginrespond(self, ucore.failed, 'ӺҰ');
    exit;
  end;

end;

procedure TKailleraClient.LeaveGame;
begin
  SendInstruction(new(ins_plv,init));
  fleavegamerequested := true;
end;

procedure TKailleraClient.SendInstruction(AInstruction: PInstruction; FreeIns: boolean = true);
var pkt: tpacket;
begin
  foutgoinginstructions.Push(ainstruction);
  if freeins then dispose(ainstruction,done);
  pkt := foutgoinginstructions.GetPacket();
  fudpclient.sendbuffer(tidbytes(pkt));
end;

procedure TKailleraClient.SendKeepAlive(Sender: TObject);
begin
  sendinstruction(new(ins_tmo,init));
end;

procedure TKailleraClient.SetChated(ACallback: TChated);
begin
  chated:=acallback;
end;

procedure TKailleraClient.SetGameAdd(ACallback: TGameAdd);
begin
  gameadd:=acallback;
end;

procedure TKailleraClient.SetGameChated(ACallback: TChated);
begin
  gamechated:=acallback;
end;

procedure TKailleraClient.SetGameClosed(ACallback: TGameClosed);
begin
  gameclosed:=acallback;
end;

procedure TKailleraClient.SetGameCreated(ACallback: TGameCreated);
begin
  gamecreated:=acallback;
end;

procedure TKailleraClient.SetGameDropped(ACallback: TGameDropped);
begin
  gamedropped:=acallback;
end;

procedure TKailleraClient.SetGameStarted(ACallback: TGameStarted);
begin
  gamestarted:=acallback;
end;

procedure TKailleraClient.SetGameStatusChanged(ACallback: TGameStatusChanged);
begin
  gamestatuschanged:=acallback;
end;

procedure TKailleraClient.SetGameStopped(ACallback: TNotifyEvent);
begin
  gamestopped:=acallback;
end;

procedure TKailleraClient.SetKickedOut(ACallback: TNotifyEvent);
begin
  kickedout:=acallback;
end;

procedure TKailleraClient.SetLoginRespond(ACallback: TLoginRespond);
begin
  loginrespond:=acallback;
end;

procedure TKailleraClient.SetMyGameJoined(ACallback: TNotifyEvent);
begin
  mygamejoined:=acallback;
end;

procedure TKailleraClient.SetMyGameLeft(ACallback: TNotifyEvent);
begin
  mygameleft:=acallback;
end;

procedure TKailleraClient.SetNetSyncWait(ACallback: TNetSyncWait);
begin
  netsyncwait:=acallback;
end;

procedure TKailleraClient.SetNotifyWindow(AWindow: THandle);
begin
  fnotifywindow := awindow;
end;

procedure TKailleraClient.SetPlayerAdd(ACallback: TPlayerAdd);
begin
  playeradd:=acallback;
end;

procedure TKailleraClient.SetPlayerJoined(ACallback: TPlayerInOut);
begin
  playerjoined:=acallback;
end;

procedure TKailleraClient.SetPlayerLeft(ACallback: TPlayerInOut);
begin
  playerleft:=acallback;
end;

procedure TKailleraClient.SetServerMessaged(ACallback: TServerMessaged);
begin
  servermessaged:=acallback;
end;

procedure TKailleraClient.SetUserAdd(ACallback: TUserAdd);
begin
  useradd:=acallback;
end;

procedure TKailleraClient.SetUserJoined(ACallback: TUserJoined);
begin
  userjoined:=acallback;
end;

procedure TKailleraClient.SetUserLeft(ACallback: TUserLeft);
begin
  userleft:=acallback;
end;

procedure TKailleraClient.StartGame;
begin
  SendInstruction(new(INS_GSA,init));
end;

var
  pop_size: integer;
  sending_transfer_size: integer;

function TKailleraClient.SyncData(AValues: PAnsiChar; ASize: integer): integer;
var
  delay: integer;
  procedure sendproc;
  var
    data: pansichar;
    idx: integer;
    ins: ins_gda;
  begin
    fgamedatasendbuffer.Push(avalues, asize);
    if fgamedatasendbuffer.Size = sending_transfer_size then
    begin
      data := fgamedatasendbuffer.getdata;
      idx := fgamedatasendcache.Find(data, sending_transfer_size);
      if idx = -1 then begin
        ins := new(ins_gda,Init(data[0], sending_transfer_size));
        fgamedatasendcache.Add(ins);
        sendinstruction(ins, false);
      end else begin
        sendinstruction(new(ins_cda,init(idx)));
      end;
      freemem(data);
    end;
  end;
  procedure recvproc;
  begin
    processinstruction;
    result := fgamedatarecvbuffer.Pop(avalues, pop_size);
    if result = 0 then
    begin
      delay := 0;
      repeat
        inc(delay);
        if delay > 5 then
        begin
          if assigned(netsyncwait) then
            netsyncwait(self, delay, 30);
          if delay > 30 then
          begin
            FGameSyncing := false;
            exit;
          end;
        end;
        processinstruction;
        result := fgamedatarecvbuffer.Pop(avalues, pop_size);
      until result <> 0;
    end;
  end;
var
  game_window: thandle;
begin
  inc(fframeno);
//  writeln('frame number: ', fframeno);
  if not FGameSyncing then begin result := 0; exit; end;
  if not fallready then
  begin
    {$ifdef DEBUG} allocconsole; {$endif}
    { gda ܰ迡   ִ ʱȭ }
    game_window := getforegroundwindow();
    bringwindowtotop(game_window);
    writeln('synctime frames: ', fignoreframecount);
    pop_size := asize * fplayers;
    writeln('pop size: ', pop_size);
    sending_transfer_size := asize * fconnection;
    writeln('sending transfer size: ', sending_transfer_size);
    writeln('recving transfer size: ', asize*fplayers*fconnection);
    { rdy ñ׳ ȯ }
    sendinstruction(new(ins_rdy,init));
    writeln('sent rdy');
    delay := 0;
    repeat
      inc(delay);
      if delay > 5 then
      begin
        if assigned(netsyncwait) then begin
          netsyncwait(self, delay, 30);
        end;
        if delay > 30 then begin FGameSyncing:= false; result:=0; exit; end;
      end;
      processinstruction;
    until fallready;
    writeln('got rdy');
    sleep(100);
  end;
  sendproc;
  if fframeno > fignoreframecount then
    recvproc
  else begin
    result:=0;
    exit;
  end;
end;

procedure TKailleraClient.TerminateGame;
begin
  dropgame;
end;

{ TListener }

constructor TListener.Create(AKailleraClient: TKailleraClient);
begin
  inherited Create(false);
  this := akailleraclient;
  setlength(RecvBuffer, MAX_PACKET_SIZE);
end;

procedure TListener.Execute;
begin
  inherited;
  while not terminated do
  begin
    while this.FUDPClient.Connected do
    begin
      this.ProcessInstruction;
    end;
    sleep(1);
  end;
end;

end.
