unit CommonUtils;

interface

uses Classes;

type

  { TDelayChecker }

  TDelayChecker = class
  private
    Pings: array of integer;
    FTime: TDateTime;
    FCount: integer;
  public
    constructor Create(PingCount: integer);
    procedure CheckIn;
    procedure CheckOut;
    property Count: integer read fcount;
    function GetPing: integer;
  end;

  { TNumberPool }

  TUsingNumber = record
    Number: integer;
    Using: boolean;
  end;

  TNumberPool = class
  private
    FUsingNumbers: array of TUsingNumber;
  public
    constructor Create(Min, Max: integer);
    function PullNumber: integer;
    procedure RestoreNumber(ANumber: integer);
    function RemainingNumbers: integer;
  end;

function DivValue(AValue: string): TStrings;

implementation

uses sysutils, dateutils;

{ TDelayChecker }

constructor TDelayChecker.Create(PingCount: integer);
begin
  setlength(pings, pingcount);
end;

procedure TDelayChecker.CheckIn;
begin
  ftime := time;
  inc(fcount);
end;

procedure TDelayChecker.CheckOut;
begin
  pings[fcount-1] := millisecondof(time - ftime);
end;

function TDelayChecker.GetPing: integer;
var
  i, sum: integer;
begin
  sum := 0;
  for i:= 0 to length(pings)-1 do
  begin
    sum := sum + pings[i];
  end;
  result := sum div length(pings);
end;

{ TNumberPool }

constructor TNumberPool.Create(Min, Max: integer);
var
  count: integer;
  i: integer;
  num: integer;
begin
  count := max - min + 1;
  setlength(fusingnumbers, count);
  num := min;
  for i:= 0 to count-1 do
  begin
    fusingnumbers[i].Number:= num;
    fusingnumbers[i].Using:= false;
    inc(num);
  end;
end;

function TNumberPool.PullNumber: integer;
var i: integer;
begin
  result:=-1;
  for i:= 0 to length(fusingnumbers)-1 do
    if not fusingnumbers[i].Using then begin
      result := fusingnumbers[i].Number;
      fusingnumbers[i].Using := true;
      break;
    end;
end;

procedure TNumberPool.RestoreNumber(ANumber: integer);
var i: integer;
begin
  for i:= 0 to length(fusingnumbers)-1 do
    if fusingnumbers[i].number = anumber then
    begin
      fusingnumbers[i].Using:=false;
      break;
    end;
end;

function TNumberPool.RemainingNumbers: integer;
var i, cnt: integer;
begin
  cnt:=0;
  for i:= 0 to length(fusingnumbers)-1 do
    if not fusingnumbers[i].Using then inc(cnt);
  result := cnt;
end;

function DivValue(AValue: string): TStrings;
begin
  result := TStringList.Create;
  result.Delimiter := ';';
  extractstrings([';'], [], pchar(avalue), result);
end;

end.
