unit Instructions;

interface

uses SysUtils, VarContainers, Types;

{ Defined Types }

type

// packet = (instruction count) + array of (packed instruction) ; TPacket
// packed instruction = (serial number) + (instruction) ; TPacked
// instruction ; TInstruction

  TInstructionID = (
//  User Leave / Left
    ULV,
//  User Joined
    UJN,
//  User Login Request
    LRQ,
//  User List
    ULS,
//  Ping
    PING,
//  Pong
    PONG,
//  Chat
    CHT,
//  Game Chat
    GCH,
//  Client Timeout
    TMO,
//  Game Create / Created
    GCR,
//  Player Leave / Left
    PLV,
//  Player Join / Joined
    PJN,
//  Player List
    PLS,
//  Game Status
    GST,
//  Kick Player
    KCK,
//  Game Closed
    GCL,
//  Game Start / Started
    GSA,
//  Game Data
    GDA,
//  Cached Game Data
    CDA,
//  Game Drop / Dropped
    DRP,
//  Player / Players Ready
    RDY,
//  Login Rejected
    RJT,
//  Message Of The Day
    SVR
  );

  INS_ULV = class; INS_UJN = class; INS_LRQ = class; INS_ULS = class;
  INS_PING = class; INS_PONG = class; INS_CHT = class; INS_GCH = class;
  INS_TMO = class; INS_GCR = class; INS_PLV = class; INS_PJN = class;
  INS_PLS = class; INS_GST = class; INS_KCK = class; INS_GCL = class;
  INS_GSA = class; INS_GDA = class; INS_CDA = class; INS_DRP = class;
  INS_RDY = class; INS_RJT = class; INS_SVR = class;

  TPacked = TByteDynArray;
  TString9 = string[9];
  TString31 = string[31];
  TString127 = string[127];

  TUserRec = record
    Name: TString31;
    Ping: LongWord;
    Status: Byte;
    Id: Word;
    Conn: Byte;
  end;

  TGameRec = record
    Title: TString127;
    Id: LongWord;
    AppName: TString127;
    Owner: TString31;
    userscount: TString9;
    Status: Byte;
  end;

  TInstruction = class
  private
    FID: TInstructionID;
    FVarQueue: TVarQueue;
  protected
    property VarQueue: TVarQueue read FVarQueue;
    property ID: TInstructionID read FID write FID;
  public
    UserName: TString31;
    constructor Create; virtual;
    destructor Destroy; override;
    function GetID: TInstructionID;
    function GetIDString: string;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
    class function CreateInstructionByID(AID: TInstructionID): TInstruction; overload;
    class function CreateInstructionByID(AID: Byte): TInstruction; overload;
  end;

  //  Instruction  ⺻  ܿ Ǹ ؼ ߰  Init  д
  // ̰Ϳ  Ķʹ stirng, integer  Ϲ Ÿ ϰ .

  INS_ULV = class(TInstruction)
  private
    FUserId: Word;
    FLeaveMessage: TString127;
  public
    constructor Create; override;
    constructor Init(ALeaveMessage: string; AUserName: string = ''; AUserID: integer = -1);
    property UserId: Word read FUserId;
    property LeaveMessage: TString127 read fleavemessage;
    function PackInstruction(ASerialNo: Word): TPacked; override;
    procedure UnpackInstruction(APacked: TPacked); override;
  end;

  { INS_UJN }

  INS_UJN = class(TInstruction)
  private
    FUserId: Word;
    FPing: LongWord;
    FConnectionType: Byte;
  public
    constructor Create; override;
    constructor Init(AUserName: String; AUserID, APing, AConnectionType: integer);
    procedure UnpackInstruction(APacked: TPacked); override;
    function PackInstruction(ASerialNo: Word): TPacked; override;
    property UserId: Word read FUserId;
    property Ping: LongWord read FPing;
    property ConnectionType: Byte read FConnectionType;
  end;

  { INS_LRQ }

  INS_LRQ = class(TInstruction) // UserLoginRequest
  private
    FAppName: TString127;
    FConnection: Byte;
  public
    constructor Create; override;
    constructor Init(AUserName, AAppName: string; AConnection: Byte);
    property AppName: TString127 read FAppName;
    property Connection: Byte read FConnection;
    function PackInstruction(ASerialNo: Word): TPacked; override;
    procedure UnpackInstruction(APacked: TPacked); override;
  end;

  TUsers = array[1..300] of TUserRec;
  TGames = array[1..100] of TGameRec;

  { INS_ULS }

  INS_ULS = class(TInstruction)
  public
    Users: TUsers;
    Games: TGames;
    UsersCount: LongWord;
    GamesCount: LongWord;
    constructor Create; override;
    procedure UnpackInstruction(APacked: TPacked); override;
    function PackInstruction(ASerialNo: Word): TPacked; override;
  end;

  { INS_PING }

  INS_PING = class(TInstruction)
  public
    constructor Create; override;
    function PackInstruction(ASerialNo: Word): TPacked; override;
  end;

  INS_PONG = class(TInstruction)
  public
    constructor Create; override;
    function PackInstruction(ASerialNo: Word): TPacked; override;
  end;

  INS_CHT = class(TInstruction)
  private
  public
    Message: TString127;
    constructor Create; override;
    constructor Init(AUserName, AMessage: string);
    function PackInstruction(ASerialNo: Word): TPacked; override;
    procedure UnpackInstruction(APacked: TPacked); override;
  end;

  INS_GCH = class(TInstruction)
  private
    FMessage: TString127;
  public
    constructor Create; override;
    property Message: TString127 read FMessage write FMessage;
    function PackInstruction(ASerialNo: Word): TPacked; override;
    procedure UnpackInstruction(APacked: TPacked); override;
  end;

  INS_TMO = class(TInstruction)
  public
    constructor Create; override;
  end;

  INS_GCR = class(TInstruction) // GameCreate/Created
  public
    GameName: TString127;
    AppName: TString127;
    GameId: LongWord;
    constructor Create; override;
    constructor Init(AUserName, AGameName, AAppName: string; AGameID: LongWord);
    function PackInstruction(ASerialNo: Word): TPacked; override;
    procedure UnpackInstruction(APacked: TPacked); override;
  end;

  INS_PLV = class(TInstruction)
  private
    FUserId: Word;
  public
    constructor Create; override;
    constructor Init(AUserName: string; AUserID: integer);
    property UserId: Word read FUserId;
    function PackInstruction(ASerialNo: Word): TPacked; override;
    procedure UnpackInstruction(APacked: TPacked); override;
  end;

  INS_PJN = class(TInstruction)
  private
    FGameID: LongWord;
    FPlayerName: TString31;
    FPing: LongWord;
    FUserID: Word;
    FConnection: Byte;
  public
    constructor Create; override;
    function PackInstruction(ASerialNo: Word): TPacked; override;
    procedure UnpackInstruction(APacked: TPacked); override;
    property GameID: LongWord read FGameID write FGameID;
    property PlayerName: TString31 read FPlayerName write FPlayerName;
    property Ping: LongWord read FPing write FPing;
    property UserID: Word read FUserID write FUserID;
    property Connection: Byte read FConnection write FConnection;
  end;

  TPlayerRec = record
    Name: TString31;
    Ping: LongWord;
    ID: Word;
    Conn: Byte;
  end;

  TPlayers = array[1..16] of TPlayerRec;
  INS_PLS = class(TInstruction)
  private
    FPlayersCount: LongWord;
  public
    Players: TPlayers;
    constructor Create; override;
    procedure UnpackInstruction(APacked: TPacked); override;
    property PlayersCount: LongWord read FPlayersCount write FPlayersCount;
  end;

  INS_GST = class(TInstruction)
  private
    FGameID: LongWord;
    FGameStatus: Byte;
    FNumPlayers: Byte;
    FMaxPlayers: Byte;
  public
    constructor Create; override;
    constructor Init(AGameID, AGameStatus, ANumPlayers, AMaxPlayers: integer);
    procedure UnpackInstruction(APacked: TPacked); override;
    function PackInstruction(ASerialNo: Word): TPacked; override;
    property GameID: LongWord read FGameID;
    property GameStatus: Byte read FGameStatus;
    property NumPlayers: Byte read FNumPlayers;
    property MaxPlayers: Byte read FMaxPlayers;
  end;

  INS_KCK = class(TInstruction)
  private
    FUserID: Word;
  public
    constructor Create; override;
    function PackInstruction(ASerialNo: Word): TPacked; override;
    property UserID: Word read FUserID write FUserID;
  end;

  INS_GCL = class(TInstruction)
  private
    FGameID: LongWord;
  public
    constructor Create; override;
    constructor Init(AGameID: integer);
    property GameID: LongWord read FGameID;
    function PackInstruction(ASerialNo: Word): TPacked; override;
    procedure UnpackInstruction(APacked: TPacked); override;
  end;

  INS_GSA = class(TInstruction)
  private
    FTimeBand: Word;
    FPlayerNo: Byte;
    FMaxPlayers: Byte;
  public
    constructor Create; override;
    property TimeBand: Word read FTimeBand write FTimeBand;
    property PlayerNo: Byte read FPlayerNo write FPlayerNo;
    property MaxPlayers: Byte read FMaxPlayers write FMaxPlayers;
    procedure UnpackInstruction(APacked: TPacked); override;
    function PackInstruction(ASerialNo: Word): TPacked; override;
  end;

  INS_GDA = class(TInstruction)
  private
  public
    Data: array[0..63] of Byte;
    Size: Word;
    constructor Create; override;
    constructor Init(const AData; ASize: Word);
    destructor Destroy; override;
    function PackInstruction(ASerialNo: Word): TPacked; override;
    procedure UnpackInstruction(APacked: TPacked); override;
  end;

  INS_CDA = class(TInstruction)
  private
    FIndex: Byte;
  public
    constructor Create; override;
    constructor Init(AIndex: integer);
    property Index: Byte read FIndex;
    function PackInstruction(ASerialNo: Word): TPacked; override;
    procedure UnpackInstruction(APacked: TPacked); override;
  end;

  INS_DRP = class(TInstruction)
  private
    FPlayerNumber: Byte;
  public
    constructor Create; override;
    property PlayerNumber: Byte read FPlayerNumber write FPlayerNumber;
    function PackInstruction(ASerialNo: Word): TPacked; override;
    procedure UnpackInstruction(APacked: TPacked); override;
  end;

  INS_RDY = class(TInstruction)
  public
    constructor Create; override;
    function PackInstruction(ASerialNo: Word): TPacked; override;
    procedure UnpackInstruction(APacked: TPacked); override;
  end;

  INS_RJT = class(TInstruction) // User Login Status
  private
    FUserId: Word;
    FMsg: TString127;
  public
    constructor Create; override;
    procedure UnpackInstruction(APacked: TPacked); override;
    property UserId: Word read FUserId;
    property Msg: TString127 read FMsg;
  end;

  INS_SVR = class(TInstruction)
  private
    FMsg: TString127;
  public
    constructor Create; override;
    procedure UnpackInstruction(APacked: TPacked); override;
    property Msg: TString127 read FMsg;
  end;

const
  IDString: array[TInstructionID] of string = (
    'ULV', 'UJN', 'LRQ', 'ULS', 'PING', 'PONG', 'CHT', 'GCH', 'TMO', 'GCR', 'PLV', 'PJN', 'PLS',
    'GST', 'KCK', 'GCL', 'GSA', 'GDA', 'CDA', 'DRP', 'RDY', 'RJT', 'SVR'
  );

implementation

uses classes;

type
  TInstructionClass = class of TInstruction;

const
  PingPongBody: packed record
    Block: array[0..3] of LongWord;
  end = ( Block: (0, 1, 2, 3) );

  IDValue: array[TInstructionID] of Byte = (
    1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23
  );

  InstructionClass: array[TInstructionID] of TInstructionClass = (
    INS_ULV, INS_UJN, INS_LRQ, INS_ULS, INS_PING, INS_PONG, INS_CHT, INS_GCH,
    INS_TMO, INS_GCR, INS_PLV, INS_PJN, INS_PLS, INS_GST, INS_KCK, INS_GCL,
    INS_GSA, INS_GDA, INS_CDA, INS_DRP, INS_RDY, INS_RJT, INS_SVR
  );

{ TInstruction }

function TInstruction.GetID: TInstructionID;
begin
  result := ID;
end;

function TInstruction.GetIDString: string;
begin
  result := IDString[ID];
end;

constructor TInstruction.Create;
begin
  FVarQueue := TVarQueue.Create;
end;

procedure TInstruction.UnpackInstruction(APacked: TPacked);
begin
  FVarQueue.SetBinaryData(APacked, length(apacked));
  username := FVarQueue.Popstring;
end;

class function TInstruction.CreateInstructionByID(
  AID: TInstructionID): TInstruction;
begin
  result := InstructionClass[AID].Create;
end;

class function TInstruction.CreateInstructionByID(AID: Byte): TInstruction;
begin
  result := CreateInstructionByID(TInstructionID(AID-1));
end;

destructor TInstruction.Destroy;
begin
  FVarQueue.Free;
  inherited;
end;

function TInstruction.PackInstruction(ASerialNo: Word): TPacked;
var vq: tvarqueue;
begin
  vq := tvarqueue.Create;
  vq.Push(ASerialNo);
  vq.Push(Word(FVarQueue.Size+1+length(username)+1)); // include insid
  vq.Push(IDValue[FID]);
  vq.Push(UserName);
  vq.Push(FVarQueue.GetBinaryData);
  result := vq.GetBinaryData;
  vq.Free;
end;

{ INS_LRQ }

constructor INS_LRQ.Init(AUserName, AAppName: string; AConnection: Byte);
begin
  create;
  username := ansistring(ausername);
  fappname := ansistring(aappname);
  fconnection := aconnection;
end;

function INS_LRQ.PackInstruction(ASerialNo: Word): TPacked;
begin
  with VarQueue do
  begin
    Push(FAppName);
    Push(FConnection);
  end;
  result := inherited packinstruction(aserialno);
end;

procedure INS_LRQ.UnpackInstruction(APacked: TPacked);
begin
  inherited UnpackInstruction(APacked);
  with varqueue do
  begin
    fappname := popstring;
    pop(fconnection);
  end;
end;

constructor INS_LRQ.Create;
begin
  inherited Create;
  ID := LRQ;
end;

{ INS_PONG }

constructor INS_PONG.Create;
begin
  inherited;
  ID := PONG;
end;

function INS_PONG.PackInstruction(ASerialNo: Word): TPacked;
begin
  FVarQueue.Push(PingPongBody, sizeof(PingPongBody));
  result := inherited packinstruction(aserialno);
end;

{ INS_PING }

constructor INS_PING.Create;
begin
  inherited;
  ID := PING;
end;

function INS_PING.PackInstruction(ASerialNo: Word): TPacked;
begin
  VarQueue.Push(PingPongBody, sizeof(PingPongBody));
  Result:=inherited PackInstruction(ASerialNo);
end;

{ INS_ULS }

constructor INS_ULS.Create;
begin
  inherited;
  ID := ULS;
  userscount := 0;
  gamescount := 0;
end;

procedure INS_ULS.UnpackInstruction(APacked: TPacked);
var i:integer;
begin
  inherited;
  with varqueue do
  begin
    pop(userscount);
    pop(gamescount);
    for i:= 1 to userscount do
      with users[i] do
      begin
        name := popstring;
        pop(Ping);
        pop(Status);
        pop(Id);
        pop(Conn);
      end;
    for i:= 1 to gamescount do
      with games[i] do
      begin
        title := popstring;
        pop(Id);
        appname := popstring;
        owner := popstring;
        userscount := popstring;
        Pop(Status);
      end;
  end;
end;

function INS_ULS.PackInstruction(ASerialNo: Word): TPacked;
var i: integer;
begin
  with varqueue do
  begin
    push(userscount);
    push(gamescount);
    for i:= 1 to userscount do
      with users[i] do
      begin
        push(name);
        push(ping);
        push(status);
        push(Id);
        push(conn);
      end;
    for i:= 1 to gamescount do
      with games[i] do
      begin
        push(title);
        push(id);
        push(appname);
        push(owner);
        push(userscount);
        push(status);
      end;
  end;
  Result:=inherited PackInstruction(ASerialNo);
end;

{ INS_GCR }

constructor INS_GCR.Create;
begin
  inherited Create;
  ID := GCR;
  GameId := LongWord(-1);
end;

constructor INS_GCR.Init(AUserName, AGameName, AAppName: string; AGameID: LongWord);
begin
  create;
  username := ansistring(ausername);
  gamename := ansistring(agamename);
  appname := ansistring(aappname);
  gameid := agameid;
end;

function INS_GCR.PackInstruction(ASerialNo: Word): TPacked;
begin
  VarQueue.Push(GameName);
  VarQueue.Push(AppName);
  VarQueue.Push(GameId);
  result := inherited packinstruction(aserialno);
end;

procedure INS_GCR.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with VarQueue do
  begin
    gamename := popstring;
    appname := popstring;
    Pop(GameId);
  end;
end;

{ INS_ULV }

constructor INS_ULV.Create;
begin
  inherited Create;
  ID := ULV;
end;

constructor INS_ULV.Init(ALeaveMessage: string; AUserName: string = ''; AUserID: integer = -1);
begin
  create;
  username := ansistring(ausername);
  fuserid := word(auserid);
  fleavemessage := ansistring(aleavemessage);
end;

function INS_ULV.PackInstruction(ASerialNo: Word): TPacked;
begin
  VarQueue.Push(FUserId);
  VarQueue.Push(Fleavemessage);
  result := inherited packinstruction(aserialno);
end;

procedure INS_ULV.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with varqueue do
  begin
    pop(fuserid);
    fleavemessage := popstring;
  end;
end;

{ INS_RJT }

constructor INS_RJT.Create;
begin
  inherited;
  Id := RJT;
end;

procedure INS_RJT.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with varqueue do
  begin
    Pop(FUserId);
    fmsg := popstring;
  end;
end;

{ INS_UJN }

constructor INS_UJN.Create;
begin
  inherited;
  ID := UJN;
end;

constructor INS_UJN.Init(AUserName: String; AUserID, APing, AConnectionType: integer);
begin
  create;
  UserName:= ansistring(ausername);
  fuserid := auserid;
  fping := aping;
  fconnectiontype := aconnectiontype;
end;

procedure INS_UJN.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  VarQueue.Pop(FUserId);
  VarQueue.Pop(FPing);
  VarQueue.Pop(FConnectionType);
end;

function INS_UJN.PackInstruction(ASerialNo: Word): TPacked;
begin
  with varqueue do
  begin
    push(fuserid);
    push(fping);
    push(fconnectiontype);
  end;
  Result:=inherited PackInstruction(ASerialNo);
end;

{ INS_SVR }

constructor INS_SVR.Create;
begin
  inherited;
  ID := SVR;
end;

procedure INS_SVR.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with VarQueue do
  begin
    fmsg := popstring;
  end;
end;

{ INS_GSA }

constructor INS_GSA.Create;
begin
  inherited Create;
  Id := GSA;
  FTimeBand := Word(-1);
  FPlayerNo := Byte(-1);
  FMaxPlayers := Byte(-1);
end;

procedure INS_GSA.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with VarQueue do
  begin
    Pop(FTimeBand);
    Pop(FPlayerNo);
    Pop(FMaxPlayers);
  end;
end;

function INS_GSA.PackInstruction(ASerialNo: Word): TPacked;
begin
  with FVarQueue do
  begin
    Push(FTimeBand);
    Push(FPlayerNo);
    Push(FMaxPlayers);
  end;
  result := inherited packinstruction(aserialno);
end;

{ INS_PLV }

constructor INS_PLV.Create;
begin
  inherited;
  ID := PLV;
  FUserId := Word(-1);
end;

constructor INS_PLV.Init(AUserName: string; AUserID: integer);
begin
  create;
  fuserid := auserid;
end;

function INS_PLV.PackInstruction(ASerialNo: Word): TPacked;
begin
  VarQueue.Push(FUserId);
  result := inherited packinstruction(aserialno);
end;

procedure INS_PLV.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with VarQueue do
  begin
    Pop(FUserID);
  end;
end;

{ INS_DRP }

constructor INS_DRP.Create;
begin
  inherited;
  ID := DRP;
  FPlayerNumber := 0;
end;

function INS_DRP.PackInstruction(ASerialNo: Word): TPacked;
begin
  with VarQueue do
  begin
    Push(FPlayerNumber);
  end;
  result := inherited packinstruction(aserialno);
end;

procedure INS_DRP.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with varqueue do
  begin
    pop(fplayernumber);
  end;
end;

{ INS_GDA }

constructor INS_GDA.Create;
begin
  inherited;
  ID := GDA;
end;

destructor INS_GDA.Destroy;
begin
  inherited;
end;

constructor INS_GDA.Init(const AData; ASize: Word);
begin
  Create;
  move(adata, Data, asize);
  Size := asize;
end;

function INS_GDA.PackInstruction(ASerialNo: Word): TPacked;
begin
  varqueue.Push(Size);
  varqueue.Push(Data, Size);
  result := inherited packinstruction(aserialno);
end;

procedure INS_GDA.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  varqueue.Pop(Size);
  varqueue.Pop(Data, Size);
end;

{ INS_CHT }

constructor INS_CHT.Create;
begin
  inherited;
  ID := CHT;
end;

constructor INS_CHT.Init(AUserName, AMessage: string);
begin
  create;
  username := ansistring(ausername);
  message := ansistring(amessage);
end;

function INS_CHT.PackInstruction(ASerialNo: Word): TPacked;
begin
  with varqueue do
  begin
    push(message);
  end;
  result := inherited packinstruction(aserialno);
end;

procedure INS_CHT.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with varqueue do
  begin
    message := popstring;
  end;
end;

{ INS_GCH }

constructor INS_GCH.Create;
begin
  inherited;
  ID := GCH;
end;

function INS_GCH.PackInstruction(ASerialNo: Word): TPacked;
begin
  varqueue.Push(fmessage);
  result := inherited packinstruction(aserialno);
end;

procedure INS_GCH.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  fmessage := varqueue.Popstring;
end;

{ INS_TMO }

constructor INS_TMO.Create;
begin
  inherited;
  ID := TMO;
end;

{ INS_PJN }

constructor INS_PJN.Create;
begin
  inherited;
  ID := PJN;
  fgameid := 0;
  fplayername := '';
  fping := 0;
  fuserid := word(-1);
  fconnection := 0;
end;

function INS_PJN.PackInstruction(ASerialNo: Word): TPacked;
begin
  with varqueue do
  begin
    push(fgameid);
    push(fplayername);
    push(fping);
    push(fuserid);
    push(fconnection);
  end;
  result := inherited packinstruction(aserialno);
end;

procedure INS_PJN.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with varqueue do
  begin
    pop(fgameid);
    fplayername := popstring;
    pop(fping);
    pop(fuserid);
    pop(fconnection);
  end;
end;

{ INS_PLS }

constructor INS_PLS.Create;
begin
  inherited;
  ID := PLS;
end;

procedure INS_PLS.UnpackInstruction(APacked: TPacked);
var i:integer;
begin
  inherited;
  with varqueue do
  begin
    pop(fplayerscount);
    for i:= 1 to fplayerscount do
    begin
      players[i].name := popstring;
      pop(players[i].ping);
      pop(players[i].id);
      pop(players[i].conn);
    end;
  end;
end;

{ INS_GST }

constructor INS_GST.Create;
begin
  inherited;
  ID := GST;
end;

constructor INS_GST.Init(AGameID, AGameStatus, ANumPlayers, AMaxPlayers: integer);
begin
  create;
  fgameid := agameid;
  fgamestatus := agamestatus;
  fnumplayers := anumplayers;
  fmaxplayers := amaxplayers;
end;

function INS_GST.PackInstruction(ASerialNo: Word): TPacked;
begin
  with varqueue do
  begin
    push(fgameid);
    push(fgamestatus);
    push(fnumplayers);
    push(fmaxplayers);
  end;
  result := inherited packinstruction(aserialno);
end;

procedure INS_GST.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with varqueue do
  begin
    pop(fgameid);
    pop(fgamestatus);
    pop(fnumplayers);
    pop(fmaxplayers);
  end;
end;

{ INS_KCK }

constructor INS_KCK.Create;
begin
  inherited;
  ID := KCK;
end;

function INS_KCK.PackInstruction(ASerialNo: Word): TPacked;
begin
  with varqueue do
    push(fuserid);
  result := inherited packinstruction(aserialno);
end;

{ INS_GCL }

constructor INS_GCL.Create;
begin
  inherited;
  ID := GCL;
end;

constructor INS_GCL.Init(AGameID: integer);
begin
  create;
  fgameid := agameid;
end;

function INS_GCL.PackInstruction(ASerialNo: Word): TPacked;
begin
  varqueue.Push(fgameid);
  result := inherited packinstruction(aserialno);
end;

procedure INS_GCL.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with varqueue do
  begin
    pop(fgameid);
  end;
end;

{ INS_CDA }

constructor INS_CDA.Create;
begin
  inherited;
  ID := CDA;
end;

constructor INS_CDA.Init(AIndex: integer);
begin
  create;
  findex := aindex;
end;

function INS_CDA.PackInstruction(ASerialNo: Word): TPacked;
begin
  with varqueue do
  begin
    push(findex);
  end;
  result := inherited packinstruction(aserialno);
end;

procedure INS_CDA.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with varqueue do
  begin
    pop(findex);
  end;
end;

{ INS_RDY }

constructor INS_RDY.Create;
begin
  inherited;
  ID := RDY;
end;

function INS_RDY.PackInstruction(ASerialNo: Word): TPacked;
begin
  result := inherited packinstruction(aserialno);
end;

procedure INS_RDY.UnpackInstruction(APacked: TPacked);
begin
  inherited;
end;

end.
