unit Instructions2;

interface

uses SysUtils, Containers, Types;

{ Defined Types }

type

// packet = (instruction count) + array of (packed instruction) ; TPacket
// packed instruction = (serial number) + (instruction) ; TPacked
// instruction ; TInstruction

  TInstructionID = (
// Unknown, Not Exists Instruction
    UNK,
//  User Leave / Left
    ULV,
//  User Joined
    UJN,
//  User Login Request
    LRQ,
//  User List
    ULS,
//  Ping
    PING,
//  Pong
    PONG,
//  Chat
    CHT,
//  Game Chat
    GCH,
//  Client Timeout
    TMO,
//  Game Create / Created
    GCR,
//  Player Leave / Left
    PLV,
//  Player Join / Joined
    PJN,
//  Player List
    PLS,
//  Game Status
    GST,
//  Kick Player
    KCK,
//  Game Closed
    GCL,
//  Game Start / Started
    GSA,
//  Game Data
    GDA,
//  Cached Game Data
    CDA,
//  Game Drop / Dropped
    DRP,
//  Player / Players Ready
    RDY,
//  Login Rejected
    RJT,
//  Message Of The Day
    SVR
  );

  INS_ULV = ^TInsULV ; INS_UJN = ^TInsUJN ; INS_LRQ = ^TInsLRQ ; INS_ULS = ^TInsULS ;
  INS_PING = ^TInsPING ; INS_PONG = ^TInsPONG ; INS_CHT = ^TInsCHT ; INS_GCH = ^TInsGCH ;
  INS_TMO = ^TInsTMO ; INS_GCR = ^TInsGCR ; INS_PLV = ^TInsPLV ; INS_PJN = ^TInsPJN ;
  INS_PLS = ^TInsPLS ; INS_GST = ^TInsGST ; INS_KCK = ^TInsKCK ; INS_GCL = ^TInsGCL ;
  INS_GSA = ^TInsGSA ; INS_GDA = ^TInsGDA ; INS_CDA = ^TInsCDA ; INS_DRP = ^TInsDRP ;
  INS_RDY = ^TInsRDY ; INS_RJT = ^TInsRJT ; INS_SVR = ^TInsSVR ; INS_UNK = ^TInsUNK;

  TPacked = TByteDynArray;
  TString9 = string[9];
  TString31 = string[31];
  TString127 = string[127];

  PInstruction = ^TInstruction;
  TInstruction = object
  private
    FID: TInstructionID;
    FByteBuffer: TByteBuffer;
  protected
    property ByteBuffer: TByteBuffer read FByteBuffer;
    property ID: TInstructionID read FID write FID;
  public
    UserName: TString31;
    constructor Init;
    destructor Done; virtual;
    function GetID: TInstructionID;
    function GetIDString: string;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
  end;

  //  Instruction  ⺻  ܿ Ǹ ؼ ߰  Init  д
  // ̰Ϳ  Ķʹ stirng, integer  Ϲ Ÿ ϰ .

  TInsULV = object(TInstruction)
  private
    FUserId: Word;
    FLeaveMessage: TString127;
  public
    constructor Init; overload;
    constructor Init(ALeaveMessage: string; AUserName: string = ''; AUserID: integer = -1); overload;
    destructor Done; virtual;
    property UserId: Word read FUserId;
    property LeaveMessage: TString127 read fleavemessage;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
  end;

  { TInsUJN }

  TInsUJN = object(TInstruction)
  private
    FUserId: Word;
    FPing: LongWord;
    FConnectionType: Byte;
  public
    constructor Init; overload;
    constructor Init(AUserName: String; AUserID, APing, AConnectionType: integer); overload;
    destructor Done; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    property UserId: Word read FUserId;
    property Ping: LongWord read FPing;
    property ConnectionType: Byte read FConnectionType;
  end;

  { TInsLRQ }

  TInsLRQ = object(TInstruction) // UserLoginRequest
  private
    FAppName: TString127;
    FConnection: Byte;
  public
    constructor Init; overload;
    constructor Init(AUserName, AAppName: string; AConnection: Byte); overload;
    destructor Done; virtual;
    property AppName: TString127 read FAppName;
    property Connection: Byte read FConnection;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
  end;

  TUserRec = record
    Name: TString31;
    Ping: LongWord;
    Status: Byte;
    Id: Word;
    Conn: Byte;
  end;

  TGameRec = record
    Title: TString127;
    Id: LongWord;
    AppName: TString127;
    Owner: TString31;
    userscount: TString9;
    Status: Byte;
  end;

  TUsers = array[1..300] of TUserRec;
  TGames = array[1..100] of TGameRec;

  { TInsULS }

  TInsULS = object(TInstruction)
  public
    Users: TUsers;
    Games: TGames;
    UsersCount: LongWord;
    GamesCount: LongWord;
    constructor Init;
    destructor Done; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    procedure AddUser(Name: string; Ping, Status, AUserId, Conn: integer);
  end;

  { TInsPING }

  TInsPING = object(TInstruction)
  public
    constructor Init;
    destructor Done; virtual;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
  end;

  TInsPONG = object(TInstruction)
  public
    constructor Init;
    destructor Done; virtual;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
  end;

  TInsCHT = object(TInstruction)
  public
    Message: TString127;
    constructor Init; overload;
    constructor Init(AMessage: string; AUserName: string = ''); overload;
    destructor Done; virtual;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
  end;

  TInsGCH = object(TInstruction)
  private
    FMessage: TString127;
  public
    constructor Init; overload;
    constructor Init(AMessage: string; AUserName: string = ''); overload;
    destructor Done; virtual;
    property Message: TString127 read FMessage;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
  end;

  TInsTMO = object(TInstruction)
  public
    constructor Init;
    destructor Done; virtual;
  end;

  TInsGCR = object(TInstruction) // GameCreate/Created
  public
    GameName: TString127;
    AppName: TString127;
    GameId: LongWord;
    constructor Init; overload;
    constructor Init(AGameName: string; AAppName: string = ''; AGameID: integer = -1; AUserName: string = ''); overload;
    destructor Done; virtual;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
  end;

  TInsPLV = object(TInstruction)
  private
    FUserId: Word;
  public
    constructor Init; overload;
    constructor Init(AUserName: string; AUserID: integer); overload;
    destructor Done; virtual;
    property UserId: Word read FUserId;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
  end;

  TInsPJN = object(TInstruction)
  private
    FGameID: LongWord;
    FPlayerName: TString31;
    FPing: LongWord;
    FUserID: Word;
    FConnection: Byte;
  public
    constructor Init; overload;
    constructor Init(AConnectionType: integer; AGameID: integer = 0; APlayerName: string = ''; APing: integer = 0; AUserID: integer = -1); overload;
    destructor Done; virtual;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
    property GameID: LongWord read FGameID;
    property PlayerName: TString31 read FPlayerName;
    property Ping: LongWord read FPing;
    property UserID: Word read FUserID;
    property Connection: Byte read FConnection;
  end;

  TPlayerRec = record
    Name: TString31;
    Ping: LongWord;
    ID: Word;
    Conn: Byte;
  end;

  TPlayers = array[1..16] of TPlayerRec;
  TInsPLS = object(TInstruction)
  private
    FPlayersCount: LongWord;
  public
    Players: TPlayers;
    constructor init;
    destructor Done; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
    property PlayersCount: LongWord read FPlayersCount write FPlayersCount;
  end;

  TInsGST = object(TInstruction)
  private
    FGameID: LongWord;
    FGameStatus: Byte;
    FNumPlayers: Byte;
    FMaxPlayers: Byte;
  public
    constructor Init; overload;
    constructor Init(AGameID, AGameStatus, ANumPlayers, AMaxPlayers: integer); overload;
    destructor Done; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    property GameID: LongWord read FGameID;
    property GameStatus: Byte read FGameStatus;
    property NumPlayers: Byte read FNumPlayers;
    property MaxPlayers: Byte read FMaxPlayers;
  end;

  TInsKCK = object(TInstruction)
  private
    FUserID: Word;
  public
    constructor Init; overload;
    constructor Init(AUserID: integer); overload;
    destructor Done; virtual;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    property UserID: Word read FUserID;
  end;

  TInsGCL = object(TInstruction)
  private
    FGameID: LongWord;
  public
    constructor Init; overload;
    constructor Init(AGameID: integer); overload;
    destructor Done; virtual;
    property GameID: LongWord read FGameID;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
  end;

  TInsGSA = object(TInstruction)
  private
    FTimeBand: Word;
    FPlayerNo: Byte;
    FMaxPlayers: Byte;
  public
    constructor Init(ATimeBand: integer = -1; APlayerNo: integer = -1; AMaxPlayers: integer = -1);
    destructor Done; virtual;
    property TimeBand: Word read FTimeBand;
    property PlayerNo: Byte read FPlayerNo;
    property MaxPlayers: Byte read FMaxPlayers;
    procedure UnpackInstruction(APacked: TPacked); virtual;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
  end;

  TInsGDA = object(TInstruction)
  private
  public
    Data: PAnsiChar;
    Size: Word;
    constructor Init; overload;
    constructor Init(const AData; ASize: Word); overload;
    destructor Done; virtual;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
  end;

  TInsCDA = object(TInstruction)
  private
    FIndex: Byte;
  public
    constructor Init; overload;
    constructor Init(AIndex: integer); overload;
    destructor Done; virtual;
    property Index: Byte read FIndex;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
  end;

  TInsDRP = object(TInstruction)
  private
    FPlayerNumber: Byte;
  public
    constructor Init;
    destructor Done; virtual;
    property PlayerNumber: Byte read FPlayerNumber write FPlayerNumber;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
  end;

  TInsRDY = object(TInstruction)
  public
    constructor Init;
    destructor Done; virtual;
    function PackInstruction(ASerialNo: Word): TPacked; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
  end;

  TInsRJT = object(TInstruction) // User Login Status
  private
    FUserId: Word;
    FMsg: TString127;
  public
    constructor Init;
    destructor Done; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
    property UserId: Word read FUserId;
    property Msg: TString127 read FMsg;
  end;

  TInsSVR = object(TInstruction)
  private
    FMsg: TString127;
  public
    constructor Init;
    destructor Done; virtual;
    procedure UnpackInstruction(APacked: TPacked); virtual;
    property Msg: TString127 read FMsg;
  end;

  TInsUNK = object(TInstruction)
    constructor Init;
    destructor Done; virtual;
  end;

const
  IDString: array[TInstructionID] of string = (
    'UNK',
    'ULV', 'UJN', 'LRQ', 'ULS', 'PING', 'PONG', 'CHT', 'GCH', 'TMO', 'GCR', 'PLV', 'PJN', 'PLS',
    'GST', 'KCK', 'GCL', 'GSA', 'GDA', 'CDA', 'DRP', 'RDY', 'RJT', 'SVR'
  );

function CreateInstructionByID(AID: TInstructionID): PInstruction; overload;
function CreateInstructionByID(AID: Byte): PInstruction; overload;

implementation

uses classes;

const
  PingPongBody: packed record
    Block: array[0..3] of LongWord;
  end = ( Block: (0, 1, 2, 3) );

  IDValue: array[TInstructionID] of Byte = (
    0,
    1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23
  );

function CreateInstructionByID(AID: TInstructionID): PInstruction;
begin
  case AID of
    ULV: result := new(INS_ULV, Init);
    UJN: result := new(INS_UJN, Init);
    LRQ: result := new(INS_LRQ, Init);
    ULS: result := new(INS_ULS, Init);
    PING: result := new(INS_PING, Init);
    PONG: result := new(INS_PONG, Init);
    CHT: result := new(INS_CHT, Init);
    GCH: result := new(INS_GCH, Init);
    TMO: result := new(INS_TMO, Init);
    GCR: result := new(INS_GCR, Init);
    PLV: result := new(INS_PLV, Init);
    PJN: result := new(INS_PJN, Init);
    PLS: result := new(INS_PLS, Init);
    GST: result := new(INS_GST, Init);
    KCK: result := new(INS_KCK, Init);
    GCL: result := new(INS_GCL, Init);
    GSA: result := new(INS_GSA, Init);
    GDA: result := new(INS_GDA, Init);
    CDA: result := new(INS_CDA, Init);
    DRP: result := new(INS_DRP, Init);
    RDY: result := new(INS_RDY, Init);
    RJT: result := new(INS_RJT, Init);
    SVR: result := new(INS_SVR, Init);
    else result := new(INS_UNK, Init);
  end;
end;

function CreateInstructionByID(AID: Byte): PInstruction;
begin
  result := CreateInstructionByID(TInstructionID(AID));
end;

{ TInstruction }

function TInstruction.GetID: TInstructionID;
begin
  result := ID;
end;

function TInstruction.GetIDString: string;
begin
  result := IDString[ID];
end;

constructor TInstruction.Init;
begin
  fbytebuffer.Init;
end;

procedure TInstruction.UnpackInstruction(APacked: TPacked);
begin
  FByteBuffer.SetBinaryData(APacked[0], length(apacked));
  username := FByteBuffer.Popstring;
end;

destructor TInstruction.Done;
begin
  FByteBuffer.Done;
end;

function TInstruction.PackInstruction(ASerialNo: Word): TPacked;
var buf: tbytebuffer;
begin
  buf.Init;
  buf.Push(ASerialNo);
  buf.Push(Word(FByteBuffer.Size+1+length(username)+1)); // include insid
  buf.Push(IDValue[FID]);
  buf.Push(UserName);
  buf.Push(FByteBuffer.GetBinaryData);
  result := buf.GetBinaryData;
  buf.Done;
end;

{ TInsULV }

destructor TInsULV.Done;
begin
  inherited done;
end;

constructor TInsULV.Init(ALeaveMessage, AUserName: string; AUserID: integer);
begin
  init;
  username := ansistring(ausername);
  fuserid := word(auserid);
  fleavemessage := ansistring(aleavemessage);
end;

constructor TInsULV.Init;
begin
  inherited Init;
  fid := ulv;
end;

function TInsULV.PackInstruction(ASerialNo: Word): TPacked;
begin
  ByteBuffer.Push(FUserId);
  ByteBuffer.Push(Fleavemessage);
  result := inherited packinstruction(aserialno);
end;

procedure TInsULV.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with ByteBuffer do
  begin
    pop(fuserid);
    fleavemessage := popstring;
  end;
end;

{ INS_LRQ }

destructor TInsLRQ.Done;
begin
  inherited Done;
end;

constructor TInsLRQ.Init(AUserName, AAppName: string; AConnection: Byte);
begin
  init;
  username := ansistring(ausername);
  fappname := ansistring(aappname);
  fconnection := aconnection;
end;

constructor TInsLRQ.Init;
begin
  inherited init;
  fid := lrq;
end;

function TINSLRQ.PackInstruction(ASerialNo: Word): TPacked;
begin
  with ByteBuffer do
  begin
    Push(FAppName);
    Push(FConnection);
  end;
  result := inherited packinstruction(aserialno);
end;

procedure TINSLRQ.UnpackInstruction(APacked: TPacked);
begin
  inherited UnpackInstruction(APacked);
  with ByteBuffer do
  begin
    fappname := popstring;
    pop(fconnection);
  end;
end;

{ TInsPONG }

destructor TInsPONG.Done;
begin
  inherited Done;
end;

constructor TInsPONG.Init;
begin
  inherited Init;
  fid := pong;
end;

function TInsPONG.PackInstruction(ASerialNo: Word): TPacked;
begin
  FByteBuffer.Push(PingPongBody, sizeof(PingPongBody));
  result := inherited packinstruction(aserialno);
end;

{ TInsPING }

destructor TInsPING.Done;
begin
  inherited Done;
end;

constructor TInsPING.Init;
begin
  inherited Init;
  fid := ping;
end;

function TInsPING.PackInstruction(ASerialNo: Word): TPacked;
begin
  ByteBuffer.Push(PingPongBody, sizeof(PingPongBody));
  Result:=inherited PackInstruction(ASerialNo);
end;

{ TInsULS }

procedure TInsULS.UnpackInstruction(APacked: TPacked);
var i:integer;
begin
  inherited;
  with ByteBuffer do
  begin
    pop(userscount);
    pop(gamescount);
    for i:= 1 to userscount do
      with users[i] do
      begin
        name := popstring;
        pop(Ping);
        pop(Status);
        pop(Id);
        pop(Conn);
      end;
    for i:= 1 to gamescount do
      with games[i] do
      begin
        title := popstring;
        pop(Id);
        appname := popstring;
        owner := popstring;
        userscount := popstring;
        Pop(Status);
      end;
  end;
end;

procedure TInsULS.AddUser(Name: string; Ping, Status, AUserId, Conn: integer);
begin
  inc(userscount);
  users[userscount].Name := ansistring(name);
  users[userscount].Ping := ping;
  users[userscount].Status := status;
  users[userscount].Id := AUserid;
  users[userscount].Conn := conn;
end;

destructor TInsULS.Done;
begin
  inherited Done;
end;

constructor TInsULS.Init;
begin
  inherited Init;
  fid := uls;
  userscount := 0;
  gamescount := 0;
end;

function TInsULS.PackInstruction(ASerialNo: Word): TPacked;
var i: integer;
begin
  with ByteBuffer do
  begin
    push(userscount);
    push(gamescount);
    for i:= 1 to userscount do
      with users[i] do
      begin
        push(name);
        push(ping);
        push(status);
        push(Id);
        push(conn);
      end;
    for i:= 1 to gamescount do
      with games[i] do
      begin
        push(title);
        push(id);
        push(appname);
        push(owner);
        push(userscount);
        push(status);
      end;
  end;
  Result:=inherited PackInstruction(ASerialNo);
end;

{ TInsGCR }

destructor TInsGCR.Done;
begin

  inherited Done;
end;

constructor TInsGCR.Init;
begin
  inherited init;
  fid := gcr;
end;

constructor TInsGCR.Init(AGameName, AAppName: string; AGameID: integer;
  AUserName: string);
begin
  init;
  username := ansistring(ausername);
  gamename := ansistring(agamename);
  appname := ansistring(aappname);
  gameid := agameid;
end;

function TInsGCR.PackInstruction(ASerialNo: Word): TPacked;
begin
  ByteBuffer.Push(GameName);
  ByteBuffer.Push(AppName);
  ByteBuffer.Push(GameId);
  result := inherited packinstruction(aserialno);
end;

procedure TInsGCR.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with ByteBuffer do
  begin
    gamename := popstring;
    appname := popstring;
    Pop(GameId);
  end;
end;

{ TInsRJT }

destructor TInsRJT.Done;
begin

  inherited Done;
end;

constructor TInsRJT.Init;
begin
  inherited init;
  fid := rjt;
end;

procedure TInsRJT.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with ByteBuffer do
  begin
    Pop(FUserId);
    fmsg := popstring;
  end;
end;

{ TInsUJN }

destructor TInsUJN.Done;
begin
  inherited done;
end;

constructor TInsUJN.Init(AUserName: String; AUserID, APing, AConnectionType: integer);
begin
  init;
  UserName:= ansistring(ausername);
  fuserid := auserid;
  fping := aping;
  fconnectiontype := aconnectiontype;
end;

procedure TInsUJN.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  ByteBuffer.Pop(FUserId);
  ByteBuffer.Pop(FPing);
  ByteBuffer.Pop(FConnectionType);
end;

constructor TInsUJN.Init;
begin
  inherited init;
  fid := ujn;
end;

function TInsUJN.PackInstruction(ASerialNo: Word): TPacked;
begin
  with ByteBuffer do
  begin
    push(fuserid);
    push(fping);
    push(fconnectiontype);
  end;
  Result:=inherited PackInstruction(ASerialNo);
end;

{ TInsSVR }

destructor TInsSVR.Done;
begin

  inherited Done;
end;

constructor TInsSVR.Init;
begin
  inherited init;
  fid := svr;
end;

procedure TInsSVR.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with ByteBuffer do
  begin
    fmsg := popstring;
  end;
end;

{ TInsGSA }

procedure TInsGSA.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with ByteBuffer do
  begin
    Pop(FTimeBand);
    Pop(FPlayerNo);
    Pop(FMaxPlayers);
  end;
end;

destructor TInsGSA.Done;
begin

  inherited Done;
end;

constructor TInsGSA.Init(ATimeBand, APlayerNo, AMaxPlayers: integer);
begin
  inherited init;
  fid := gsa;
  FTimeBand := atimeband;
  FPlayerNo := aplayerno;
  FMaxPlayers := amaxplayers;
end;

function TInsGSA.PackInstruction(ASerialNo: Word): TPacked;
begin
  with ByteBuffer do
  begin
    Push(FTimeBand);
    Push(FPlayerNo);
    Push(FMaxPlayers);
  end;
  result := inherited packinstruction(aserialno);
end;

{ TInsPLV }

destructor TInsPLV.Done;
begin

  inherited Done;
end;

constructor TInsPLV.Init(AUserName: string; AUserID: integer);
begin
  init;
  username := ansistring(ausername);
  fuserid := auserid;
end;

constructor TInsPLV.Init;
begin
  inherited init;
  fid := plv;
  fuserid := word(-1);
end;

function TInsPLV.PackInstruction(ASerialNo: Word): TPacked;
begin
  ByteBuffer.Push(FUserId);
  result := inherited packinstruction(aserialno);
end;

procedure TInsPLV.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with ByteBuffer do
  begin
    Pop(FUserID);
  end;
end;

{ TInsDRP }

destructor TInsDRP.Done;
begin

  inherited Done;
end;

constructor TInsDRP.Init;
begin
  inherited init;
  fid := drp;
  FPlayerNumber := 0;
end;

function TInsDRP.PackInstruction(ASerialNo: Word): TPacked;
begin
  with ByteBuffer do
  begin
    Push(FPlayerNumber);
  end;
  result := inherited packinstruction(aserialno);
end;

procedure TInsDRP.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with ByteBuffer do
  begin
    pop(fplayernumber);
  end;
end;

{ TInsGDA }

destructor TInsGDA.Done;
begin
  freemem(data, size);
  inherited Done;
end;

constructor TInsGDA.Init(const AData; ASize: Word);
begin
  init;
  getmem(data, asize);
  move(adata, Data^, asize);
  Size := asize;
end;

constructor TInsGDA.Init;
begin
  inherited init;
  fid := gda;
end;

function TInsGDA.PackInstruction(ASerialNo: Word): TPacked;
begin
  ByteBuffer.Push(Size);
  ByteBuffer.Push(Data^, Size);
  result := inherited packinstruction(aserialno);
end;

procedure TInsGDA.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  ByteBuffer.Pop(Size);
  getmem(data, size);
  ByteBuffer.Pop(Data^, Size);
end;

{ TInsCHT }

destructor TInsCHT.Done;
begin

  inherited Done;
end;

constructor TInsCHT.Init(AMessage: string; AUserName: string);
begin
  init;
  username := ansistring(ausername);
  message := ansistring(amessage);
end;

constructor TInsCHT.Init;
begin
  inherited init;
  fid := cht;
end;

function TInsCHT.PackInstruction(ASerialNo: Word): TPacked;
begin
  with ByteBuffer do
  begin
    push(message);
  end;
  result := inherited packinstruction(aserialno);
end;

procedure TInsCHT.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with ByteBuffer do
  begin
    message := popstring;
  end;
end;

{ TInsGCH }

destructor TInsGCH.Done;
begin

  inherited Done;
end;

constructor TInsGCH.Init;
begin
  inherited Init;
  fid := gch;
end;

constructor TInsGCH.Init(AMessage, AUserName: string);
begin
  init;
  fmessage := ansistring(amessage);
  username := ansistring(ausername);
end;

function TInsGCH.PackInstruction(ASerialNo: Word): TPacked;
begin
  ByteBuffer.Push(fmessage);
  result := inherited packinstruction(aserialno);
end;

procedure TInsGCH.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  fmessage := ByteBuffer.Popstring;
end;

{ TInsTMO }

destructor TInsTMO.Done;
begin

  inherited Done;
end;

constructor TInsTMO.Init;
begin
  inherited init;
  fid := tmo;
end;

{ TInsPJN }

destructor TInsPJN.Done;
begin

  inherited Done;
end;

constructor TInsPJN.Init(AConnectionType: integer;  AGameID: integer; APlayerName: string; APing, AUserID: integer);
begin
  init;
  fgameid := agameid;
  fplayername := ansistring(aplayername);
  fping := aping;
  fuserid := auserid;
  fconnection := aconnectiontype;
end;

constructor TInsPJN.Init;
begin
  inherited init;
  fid := pjn;
end;

function TInsPJN.PackInstruction(ASerialNo: Word): TPacked;
begin
  with ByteBuffer do
  begin
    push(fgameid);
    push(fplayername);
    push(fping);
    push(fuserid);
    push(fconnection);
  end;
  result := inherited packinstruction(aserialno);
end;

procedure TInsPJN.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with ByteBuffer do
  begin
    pop(fgameid);
    fplayername := popstring;
    pop(fping);
    pop(fuserid);
    pop(fconnection);
  end;
end;

{ TInsPLS }

destructor TInsPLS.Done;
begin

  inherited Done;
end;

constructor TInsPLS.init;
begin
  inherited init;
  fid := pls;
end;

procedure TInsPLS.UnpackInstruction(APacked: TPacked);
var i:integer;
begin
  inherited;
  with ByteBuffer do
  begin
    pop(fplayerscount);
    for i:= 1 to fplayerscount do
    begin
      players[i].name := popstring;
      pop(players[i].ping);
      pop(players[i].id);
      pop(players[i].conn);
    end;
  end;
end;

{ TInsGST }

destructor TInsGST.Done;
begin

  inherited Done;
end;

constructor TInsGST.Init(AGameID, AGameStatus, ANumPlayers, AMaxPlayers: integer);
begin
  init;
  fgameid := agameid;
  fgamestatus := agamestatus;
  fnumplayers := anumplayers;
  fmaxplayers := amaxplayers;
end;

constructor TInsGST.Init;
begin
  inherited init;
  fid := gst;
end;

function TInsGST.PackInstruction(ASerialNo: Word): TPacked;
begin
  with ByteBuffer do
  begin
    push(fgameid);
    push(fgamestatus);
    push(fnumplayers);
    push(fmaxplayers);
  end;
  result := inherited packinstruction(aserialno);
end;

procedure TInsGST.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with ByteBuffer do
  begin
    pop(fgameid);
    pop(fgamestatus);
    pop(fnumplayers);
    pop(fmaxplayers);
  end;
end;

{ TInsKCK }

destructor TInsKCK.Done;
begin

  inherited Done;
end;

constructor TInsKCK.Init(AUserID: integer);
begin
  fuserid := auserid;
end;

constructor TInsKCK.Init;
begin
  inherited init;
  fid := kck;
end;

function TInsKCK.PackInstruction(ASerialNo: Word): TPacked;
begin
  with ByteBuffer do
    push(fuserid);
  result := inherited packinstruction(aserialno);
end;

{ TInsGCL }

destructor TInsGCL.Done;
begin

  inherited Done;
end;

constructor TInsGCL.Init(AGameID: integer);
begin
  init;
  fgameid := agameid;
end;

constructor TInsGCL.Init;
begin
  inherited init;
  fid := gcl;
end;

function TInsGCL.PackInstruction(ASerialNo: Word): TPacked;
begin
  ByteBuffer.Push(fgameid);
  result := inherited packinstruction(aserialno);
end;

procedure TInsGCL.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with ByteBuffer do
  begin
    pop(fgameid);
  end;
end;

{ TInsCDA }

destructor TInsCDA.Done;
begin

  inherited Done;
end;

constructor TInsCDA.Init(AIndex: integer);
begin
  init;
  findex := aindex;
end;

constructor TInsCDA.Init;
begin
  inherited init;
  fid := cda;
end;

function TInsCDA.PackInstruction(ASerialNo: Word): TPacked;
begin
  with ByteBuffer do
  begin
    push(findex);
  end;
  result := inherited packinstruction(aserialno);
end;

procedure TInsCDA.UnpackInstruction(APacked: TPacked);
begin
  inherited;
  with ByteBuffer do
  begin
    pop(findex);
  end;
end;

{ TInsRDY }

destructor TInsRDY.Done;
begin

  inherited Done;
end;

constructor TInsRDY.Init;
begin
  inherited init;
  fid := rdy;
end;

function TInsRDY.PackInstruction(ASerialNo: Word): TPacked;
begin
  result := inherited packinstruction(aserialno);
end;

procedure TInsRDY.UnpackInstruction(APacked: TPacked);
begin
  inherited;
end;

{ TInsUNK }

destructor TInsUNK.Done;
begin

  inherited Done;
end;

constructor TInsUNK.Init;
begin
  inherited Init;
  fid := unk;
end;

end.
