unit KailleraUtils;

interface

uses sysutils, types,
  Instructions2, Containers, contnrs;

type
  TPacket = TByteDynArray;
  TPort = Word;

  TInstructionRecvQueue = class
  private
    FInstructionList: array[Word] of PInstruction;
    FNextSerial: Word;
    ByteBuffer: TByteBuffer;
    procedure Push(AInstruction: PInstruction; Serial: Word);
  public
    constructor Create;
    destructor Destroy; override;
    function Pop: PInstruction;
    procedure ExtractData(AData: pointer; ASize: integer);
    procedure Clear;
  end;

  PPackedItem = ^TPackedItem;
  TPackedItem = record
    FPacked: TPacked;
  end;

  TInstructionSendQueue = class(TQueue)
  private
    RecvSerial: Word;
    ByteBuffer: TByteBuffer;
  protected
    procedure PushItem(AItem: Pointer); override;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Push(AInstruction: PInstruction);
    function GetPacket: TPacket;
    procedure Clear;
  end;

  TGameDataCache = class(TQueue)
  private
    function ActualIndex(VIndex: integer): integer;
    function VirtualIndex(AIndex: integer): integer;
  public
    destructor Destroy; override;
    procedure Add(Agda: pinstruction);
    function Get(AIndex: integer): ins_gda;
    function Find(const adata: pansichar; asize: integer): integer;
    procedure Clear;
  end;

  TGameDataBuffer = class(TDataBuffer)
  public
    function GetData: PAnsiChar;
  end;

//procedure GetHostPort(const Address: string; out Host: string; out Port: integer);
function gethostport(var host: string; var port: Word; const Address: string): boolean;

implementation

uses strutils, classes, log,
{$ifndef fpc}
{$ifdef VER200}
  jclpcre;
{$else}
  regexpr;
{$endif}
{$endif}

{ TInstructionQueue }

constructor TInstructionRecvQueue.Create;
begin
  clear;
  bytebuffer.init;
end;

destructor TInstructionRecvQueue.Destroy;
begin
  bytebuffer.Done;
  clear;
  inherited;
end;

procedure TInstructionRecvQueue.Push(AInstruction: PInstruction; Serial: Word);
begin
  if finstructionlist[serial] <> nil then dispose(finstructionlist[serial], done);
  finstructionlist[serial] := ainstruction;
end;

function TInstructionRecvQueue.Pop: PInstruction;
begin
  result := finstructionlist[fnextserial];
  if result <> nil then
  begin
    finstructionlist[fnextserial] := nil;
    if fnextserial = 65535 then clear
    else inc(fnextserial);
  end;
end;

procedure TInstructionRecvQueue.Clear;
var i: integer;
begin
  for i:= 0 to length(finstructionlist)-1 do
    if finstructionlist[i] <> nil then
    begin
      dispose(finstructionlist[i], done);
      finstructionlist[i] := nil;
    end;
  fnextserial := 0;
end;

procedure TInstructionRecvQueue.ExtractData(AData: pointer; ASize: integer);
var
  cnt: byte;
  i: integer;
  serial, size: word;
  packed1: tpacked;
  id: byte;
  ins: pinstruction;
begin
  ByteBuffer.Clear;
  ByteBuffer.SetBinaryData(adata^, ASize);
  ByteBuffer.Pop(cnt);
  i:=0;
  while i < cnt do
  begin
    try
      ByteBuffer.Pop(serial);
      ByteBuffer.Pop(size);
      ByteBuffer.Pop(id);
      setlength(packed1, size-1); // exclude id
      ByteBuffer.Pop(packed1);
      Ins := CreateInstructionByID(id);
      ins^.UnpackInstruction(packed1);
      self.Push(ins, serial);
    except
      on E: EContainerOutOfData do break;
    end;
    inc(i);
  end;
end;

{ TGameDataCache }

function TGameDataCache.ActualIndex(VIndex: integer): integer;
begin
  result := Count-VIndex-1;
end;

procedure TGameDataCache.Clear;
begin
  while count <> 0 do
    dispose(ins_gda(pop), done);
end;

destructor TGameDataCache.Destroy;
begin
  clear;
  inherited;
end;

function TGameDataCache.Find(const adata: pansichar; asize: integer): integer;
var
  i:integer;
  item: ins_gda;
begin
  result := -1;
  i:=0;
  while i < list.Count do
  begin
    item := list[i];
    if comparemem(item.data, aData, aSize) then
    begin
      result := virtualindex(i);
      break;
    end;
    inc(i);
  end;
end;

function TGameDataCache.Get(AIndex: integer): ins_gda;
begin
  result := ins_gda(List.Items[ActualIndex(AIndex)]);
end;

procedure TGameDataCache.Add(Agda: pinstruction);
begin
  if Count = 256 then dispose(pinstruction(pop),done);
  Push(agda);
end;

function TGameDataCache.VirtualIndex(AIndex: integer): integer;
begin
  result := Count - (aindex + 1);
end;

{ TGameDataBuffer }

//procedure GetHostPort(const Address: string; out Host: string; out Port: integer);
//var
//  colon: integer;
//begin
//	colon := Pos(':', Address);
//	if colon <> 0 then
//	begin
//		host := midstr(Address, 1, colon-1);
//		port := StrToIntDef(MidStr(Address, colon+1, Length(Address)), 27888);
//	end
//	else
//	begin
//		host := Address;
//		port := 27888;
//	end;
//end;

function gethostport(var host: string; var port: Word; const Address: string): boolean;
var
{$IFDEF FPC}
  r: tregexprengine;
  idx, len: integer;
{$ELSE}
{$ifdef VER200}
  re: tjclregex;
{$endif}
{$ENDIF}
  sl: tstringlist;
begin
  result := true;
{$IFDEF FPC}
  generateregexprengine('[\w\-]+\.[\w\-]+(\.[\w\-]+)*(:\d)?',[],r);
  if not regexprpos(r, pchar(AAddress), idx, len) then
     result := false;
  destroyregexprengine(r);
{$ELSE}
{$ifdef VER200}
  re := tjclregex.Create;
  re.Compile('[\w\-]+\.[\w\-]+(\.[\w\-]+)*(:\d)?', false);
  if not (re.Match(Address) or ansicontainstext(address, 'localhost')) then
    result := false;
  re.Free;
{$else}
  result := execregexpr('[\w\-]+\.[\w\-]+(\.[\w\-]+)*(:\d)?', address) or ansicontainstext(address, 'localhost');
{$endif}
{$ENDIF}
  if not result then exit;
  sl := tstringlist.Create;;
  sl.Delimiter:= ':';
  sl.DelimitedText:= Address;
  host := sl[0];
  if sl.Count = 2 then
    port := strtoint(sl[1])
  else port := 27888;
  sl.Free;
end;

function TGameDataBuffer.GetData: PAnsiChar;
var sz: integer;
begin
  sz := size;
  getmem(result, sz);
  pop(result, sz);
end;

{ TInstructionStack2 }

procedure TInstructionSendQueue.Clear;
var pi: ppackeditem;
begin
  while List.Count <> 0 do
  begin
    pi := popitem;
    pi^.FPacked := nil;
    dispose(pi);
  end;
  recvserial := 0;
end;

constructor TInstructionSendQueue.Create;
begin
  inherited Create;
  ByteBuffer.Init;
  recvserial := 0;
end;

destructor TInstructionSendQueue.Destroy;
begin
  clear;
  ByteBuffer.Done;
  inherited;
end;

function TInstructionSendQueue.GetPacket: TPacket;
var
  i: integer;
begin
  ByteBuffer.Clear;
  ByteBuffer.Push(byte(list.Count));
  for i:= 0 to list.Count-1 do
  begin
    ByteBuffer.push(ppackeditem(list.Items[i])^.FPacked);
  end;
  result := ByteBuffer.GetBinaryData;
end;

procedure TInstructionSendQueue.Push(AInstruction: PInstruction);
var pi: ppackeditem;
begin
  new(pi);
  pi^.FPacked := ainstruction^.PackInstruction(RecvSerial);
  pushitem(pi);
  inc(recvserial);
end;

procedure TInstructionSendQueue.PushItem(AItem: Pointer);
var pi: ppackeditem;
begin
  inherited;
  if list.Count > 3 then
  begin
    pi := popitem;
    pi^.FPacked := nil;
    dispose(pi);
  end;
end;

end.

