unit Timer;

interface

uses
  Classes, SysUtils;

  { TTimer }

type

  TTimer = class
  private
    FOnTimer: TNotifyEvent;
    FInterval: cardinal;
    FTime: cardinal;
    FPerformEvent: boolean;
    procedure SetEnabled(const AValue: boolean);
  public
    constructor Create;
    destructor Destroy; override;
    property Enabled: boolean write SetEnabled;
    property OnTimer: TNotifyEvent write FOnTimer;
    property Interval: cardinal write FInterval;
    procedure ResetTimer;
    procedure IncTime(AMSec: cardinal);
  end;

procedure Run(AMSec: cardinal);

implementation

uses log;

var
  TimerList: TThreadList;

procedure Run(AMSec: cardinal);
var i: integer; ls:tlist;
begin
  ls := timerlist.LockList;
  //code begin
  i:=0;
  while i < ls.Count do
  begin
    ttimer(ls[i]).IncTime(amsec);
    inc(i);
  end;
  //code end
  timerlist.UnlockList;
end;

{ TTimer }

procedure TTimer.SetEnabled(const AValue: boolean);
begin
  fperformevent := avalue;
end;

constructor TTimer.Create;
begin
  with timerlist.LockList do Add(self);
  timerlist.UnlockList;
end;

destructor TTimer.Destroy;
begin
  with timerlist.locklist do remove(self);
  timerlist.UnlockList;
  inherited Destroy;
end;

procedure TTimer.IncTime(AMSec: cardinal);
begin
  inc(ftime, amsec);
  if (ftime >= finterval) then
  begin
    ftime := 0;
    if assigned(fontimer) and fperformevent then
      fontimer(self);
  end;
end;

procedure TTimer.ResetTimer;
begin
  ftime := 0;
end;

initialization
  timerlist := tthreadlist.Create;

finalization
  timerlist.Free;

end.

