unit KailleraServer;

{$ifdef fpc}
{$mode delphi}
{$endif}

interface

uses
  idudpserver, idglobal, idsockethandle,
  kaillerautils, commonutils, idcomponent, users;

type

  { TKailleraServer }

  TKailleraServer = class
  private
    FGatePort: TPort;
    FMinConnectionPort: TPort;
    FMaxConnectionPort: TPort;
    FUDPServer: TIdUDPServer;
    FPortPool: TNumberPool;
    UserManager: TUserManager;
    procedure GetConnection(AThread: TIdUDPListenerThread; AData: TIdBytes; ABinding: TIdSocketHandle);
    procedure UDPServerStatus(ASender: TObject; const AStatus: TIdStatus; const AStatusText: string);
  public
    constructor Create;
    destructor Destroy; override;
  end;

implementation

uses
  sysutils, log,
  games, idstack;

{ TKailleraServer }

procedure TKailleraServer.GetConnection(AThread: TIdUDPListenerThread;
  AData: TIdBytes; ABinding: TIdSocketHandle);
var
  recvstr: string;
  assign_port: integer;
begin
  writeln('port ',abinding.Port,' connection requested from ',abinding.PeerIP,':',abinding.PeerPort);
  recvstr := bytestostring(adata);
  writeln('('+recvstr+')');
  if pos('HELLO', recvstr) = 0 then begin writeln('connection invalid'); exit; end;
  assign_port := fportpool.PullNumber;
  if assign_port <> -1 then
  begin
    usermanager.AddUser.Activate(abinding.PeerIP, assign_port);
    writeln('port allowed: ', assign_port);
    abinding.Sendto(abinding.PeerIP, abinding.PeerPort, tobytes('HELLOD00D'+inttostr(assign_port)+#0));
  end else begin
    writeln('not allowed');
    abinding.Sendto(abinding.peerip, abinding.peerport, tobytes('TOO'#0));
  end;
end;

procedure TKailleraServer.UDPServerStatus(ASender: TObject;
  const AStatus: TIdStatus; const AStatusText: string);
begin
  trace;
end;

constructor TKailleraServer.Create;
begin
  FGatePort := 27888;
  fminconnectionport := 27890;
  fmaxconnectionport := 27899;
  fportpool := tnumberpool.Create(fminconnectionport, fmaxconnectionport);
  usermanager := tusermanager.Create;
  fudpserver := tidudpserver.Create;
  fudpserver.ThreadedEvent := true;
  fudpserver.OnUDPRead:= GetConnection;
  fudpserver.OnStatus := udpserverstatus;
  fudpserver.DefaultPort := fgateport;
  fudpserver.Active:= true;
end;

destructor TKailleraServer.Destroy;
begin
  usermanager.Free;
  fportpool.Free;
  fudpserver.Active:= false;
  fudpserver.Free;
  inherited Destroy;
end;

end.

